;****************************************************************************************
;*											*
;*	FAT16 patch code is supplied within GR8BIT platform				*
;*								http://www.gr8bit.ru	*
;*	Copyleft (cl) 2011-12 Eugeny Brychkov						*
;*	FAT16 Patch code ver.0.12 (c) 2006 by OKEI					*
;*											*
;*				License and T&C						*
;*											*
;*	You can use and modify its source and executable, given two conditions are	*
;*	met: (a) you keep these Copyleft/Copyright statements in the source code, 	*
;*	and you do not distribute software and resulting executable on the commercial	*
;*	basis and for the purposes, not related to GR8BIT platform, without written	*
;*	permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)			*
;*											*
;*	This patch works with storage subsystem built on PIC18F46K22 microcontroller,	*
;*	microcode level 3.0 and above, and may only be effective in DOS2 kernel mode	*
;*											*
;****************************************************************************************

; Note: FF,FF,00 in the drivetable set during its
; initialization in page 2

EXTBIO	EQU	0FFCAH		; extended BIOS entry
LF1FD	EQU	0F1FDH		; enable DOS RAM on page 0
BUFSEG	equ	03fedh		; DOS2 kernel RAM address for buffered I/O

FPATCH:
	; preserve everything
	push	af
	ex	af,af'
	push	af
	push	hl
	push	de
	push	bc
	exx
	push	hl
	push	de
	push	bc
	push	ix
	push	iy

	; display "FAT16 patch"
	call	_PRSTR
	db	"FAT16 patch ",0

	; sense for "2" key pressed to cancel FAT16 patch install
	ld	c,0
	call	SnsKbd		; read scan line 0
	and	4
	jr	nz,IPatch	; "2" was not pressed, install FAT16 patch

	; display "cancelled" is "2" key awas pressed
	call	_PRSTR
	db	"cancelled",0dh,0ah,0
	jr	NoPatch	; "2" pressed, do not installo patch

IPatch:
	LD	D,04h		;Get address of mapper support rutine
	LD	E,02h
	CALL	EXTBIO
	OR	A
	JR	Z,SEG_NO	;not found
	LD	B,0		;Primary mapper
	LD	A,01h		;System segment
	CALL	MAPPER		;JP  (HL)
	JR	NC,SEG_NO
	XOR	A
SEG_NO:
	ld	b,a		; preserve buffer segment in B

	di
	; enable RAM in bank 0
	in	a,(0a8h)
	push	af
	ld	a,(0ffffh)	; preserve current slot/subslot settings
	cpl
	push	af
	push	bc		; preserve buffer segment (B) in stack
	call	LF1FD		; enable RAM in bank 0

	LD	A,(0F2D0h)	; code segment at RAM bank 0
	OUT	(0FCh),A

	ld	hl,PATCHT
	ld	de,PATCHD	; perform patching
NxEntr:
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	inc	hl
	ld	a,(hl)
	inc	a
	inc	hl
	push	hl
	ld	l,c
	ld	h,b
	ld	b,a

	; test for end of array
	inc	hl
	ld	a,h
	or	l
	jr	z,EndArr		; end of the patch data
	dec	hl

	; B=byte count, HL=address in memory, DE=array
WrByte:
	ld	a,(de)
	ld	(hl),a
	inc	hl
	inc	de
	djnz	WrByte
	pop	hl
	jr	NxEntr

EndArr:
	pop	hl

	pop	af			; restore buffer segment in A from stack
	ld	(BUFSEG),a

	LD	A,03h
	OUT	(0FCh),A		; restore page 3 of the mapper RAM

	pop	af
	ld	(0ffffh),a
	pop	af
	out	(0a8h),a		; restore initial slot/sublost settings
;	ei	; do not enable interrupts

	; display "installed"
	call	_PRSTR
	db	"installed",0dh,0ah,0

NoPatch:
	; restore everything
	pop	iy
	pop	ix

	pop	bc
	pop	de
	pop	hl
	exx
	pop	bc
	pop	de
	pop	hl
	pop	af
	ex	af,af'
	pop	af
	ret
MAPPER:
	jp	(hl)

; FAT16 patch data starts here
PATCHD:
	; (removed _FORMAT patching)
;	db	09Ch,02h,
	db	09Ch,02h,0C3h,02Ch,03Fh,095h,0Dh,026h,031h,0Eh,029h,08Fh,0FCh,0CDh
	db	0DFh,033h,056h,018h,019h,0FDh,0CBh,020h,0DEh,018h,013h,0EDh,05Bh,0E8h,0BBh,018h
	db	0Dh,0F1h,0D1h,018h,02h,0F1h,0C5h,0F5h,018h,04h,0EDh,05Bh,0A3h,0BBh,07Ah,0A3h
	db	018h,018h,020h,0FAh,033h,033h,0C9h,0EDh,05Bh,0E8h,0BBh,0CDh,055h,0Dh,0EDh,05Bh
	db	0E4h,0BBh,0C0h,018h,0E9h,05Bh,0E6h,0BBh,07Ah,03Ch,028h,02h,0AFh,0C9h,03Dh,0C9h
	db	07Eh,018h,0F6h,02Ah,0FAh,0B9h,0CDh,09h,0Eh,0C8h,0DDh,07Eh,02Ah,0C3h,0CEh,039h
	db	02Ah,0FAh,0B9h,0CDh,09h,0Eh,0C8h,0DDh,07Eh,01Dh,0C3h,0D7h,03Ch,0F5h,0CDh,09h
	db	0Eh,028h,07h,0F1h,0CDh,086h,02Dh,0CBh,07Ah,0C9h,0F1h,0CDh,0D0h,0Dh,028h,010h
	db	0AFh,032h,0EAh,0BBh,03Eh,0F2h,011h,0FFh,0FFh,0CDh,086h,036h,028h,0F2h,018h,010h
	db	0E5h,01Ah,06Fh,013h,01Ah,067h,0EBh,021h,0F7h,0FFh,0B7h,0EDh,052h,0E1h,030h,06h
	db	011h,0FFh,0FFh,0B2h,037h,0C9h,0AFh,0C9h,0DDh,0E5h,0C5h,0E5h,0E5h,0DDh,0E1h,0FDh
	db	0CBh,029h,086h,0DDh,06Eh,016h,0DDh,066h,017h,0AFh,0EDh,052h,038h,01Eh,062h,06Bh
	db	029h,0E5h,05Ah,016h,00h,0DDh,06Eh,0Ch,0DDh,066h,0Dh,019h,0EBh,0CDh,026h,031h
	db	01h,0Bh,09h,0C1h,078h,0E6h,01h,047h,09h,0EBh,0BFh,0E1h,0C1h,0DDh,0E1h,0C9h
	db	0E5h,0D5h,011h,01Dh,00h,019h,0CBh,07Eh,0D1h,0E1h,0C9h,0CDh,039h,0Dh,0CDh,03Ch
	db	0Dh,00h,00h,00h,0CDh,0C7h,033h,0CDh,0D3h,033h,045h,031h,0CDh,048h,0Dh,0CDh
	db	082h,033h,0CDh,08Eh,033h,0CDh,06Ch,0Dh,00h,00h,00h,0CDh,042h,0Dh,00h,00h
	db	00h,0CDh,0D2h,030h,00h,0CDh,05Eh,0Dh,00h,00h,00h,095h,0Dh,0C8h,030h,035h
	db	031h,038h,0CDh,09Ah,033h,0CDh,0A4h,033h,0CDh,0AFh,033h,0CDh,0BBh,033h,0E2h,030h
	db	0CDh,051h,031h,0CDh,014h,031h,0CDh,020h,031h,0C3h,089h,030h,045h,031h,095h,0Dh
	db	095h,0Dh,00h,00h,095h,0Dh,0CCh,0FCh,02Fh,0D1h,01h,0FFh,0FFh,00h,095h,0Dh
	db	00h,00h,0CDh,031h,030h,023h,077h,05Eh,031h,0CDh,041h,030h,0CDh,013h,03Fh,064h
	db	031h,0CDh,01Bh,03Fh,04Fh,0AFh,018h,02h,029h,08Fh,010h,0FCh,047h,079h,085h,06Fh
	db	0EBh,078h,0C3h,00h,03Fh,0C3h,0E9h,030h,026h,031h,026h,031h,026h,031h,00h,00h
	db	04Ah,031h,026h,031h,04Ah,031h,0CDh,02Fh,033h,095h,0Dh,0CDh,04Ch,0Dh,0CDh,051h
	db	0Dh,00h,00h,00h,0CDh,02Dh,031h,045h,031h,095h,0Dh,0CDh,059h,0Dh,0DDh,0CBh
	db	01Dh,07Eh,03Eh,00h,020h,03h,03Ah,0EFh,03Fh,0DDh,04Eh,011h,071h,0C9h,023h,07Eh
	db	092h,0C0h,0DDh,0CBh,01Dh,07Eh,028h,02h,0AFh,0C9h,03Ah,0EFh,03Fh,023h,023h,023h
	db	0BEh,0C9h,0F5h,0DDh,07Eh,01Dh,0CBh,07Fh,03Ah,0F3h,03Fh,04Fh,0F1h,0C9h,04Fh,0F1h
	db	0C9h,07Dh,0B4h,028h,05h,0EDh,052h,0C3h,067h,034h,0DDh,06Eh,020h,0DDh,066h,021h
	db	0DDh,07Eh,022h,0B7h,0EDh,052h,0DEh,00h,0Dh,0CAh,070h,034h,0CBh,03Fh,0CBh,01Ch
	db	0CBh,01Dh,018h,0F4h,0EDh,042h,0F5h,0D5h,01Bh,01Bh,01Bh,0EBh,07Eh,03Dh,087h,021h
	db	025h,0BAh,085h,06Fh,07Eh,023h,066h,0C6h,01Dh,06Fh,03Eh,00h,08Ch,0CBh,07Eh,0EBh
	db	0D1h,028h,05h,0F1h,0C1h,0C3h,04Ch,026h,0F1h,0D5h,013h,013h,013h,01Ah,0EDh,04Bh
	db	0EEh,03Fh,099h,0D1h,0C1h,03Ch,0C2h,065h,026h,07Dh,080h,030h,03h,03Eh,00h,08Ch
	db	0C3h,052h,026h,0CDh,0B6h,02Dh,03Ah,0EEh,03Fh,032h,0F0h,03Fh,0C9h,013h,07Ah,0B3h
	db	020h,07h,03Ah,0F0h,03Fh,03Ch,032h,0F0h,03Fh,03Ah,0E1h,0BBh,0C9h,0AFh,032h,0EEh
	db	03Fh,0C3h,099h,025h,0CDh,09h,0Eh,028h,06h,078h,0FEh,010h,0C3h,0DAh,02Dh,0CDh
	db	0D0h,0Dh,028h,012h,03Eh,0FFh,032h,0EAh,0BBh,03Eh,0F2h,011h,0FFh,0FFh,0CDh,086h
	db	036h,028h,0F6h,0C3h,035h,02Eh,0E5h,079h,012h,013h,078h,012h,0C3h,0Fh,02Eh,022h
	db	0B4h,0BBh,0D0h,03Ah,0EEh,03Fh,03Ch,032h,0EEh,03Fh,0C9h,0FDh,034h,035h,0C0h,018h
	db	0F2h,0AFh,032h,0EFh,03Fh,0C3h,06Ah,02Bh,0AFh,032h,0EEh,03Fh,045h,05h,01Bh,0C9h
	db	0EDh,04Bh,0E8h,0BBh,03h,078h,0B1h,06h,01h,028h,0Ah,03Ah,0F0h,03Fh,018h,06h
	db	03Ah,0EEh,03Fh,018h,01h,0AFh,032h,0EFh,03Fh,0C3h,078h,02Bh,0F5h,03Ah,0EEh,03Fh
	db	032h,0F3h,03Fh,0EDh,05Bh,0B4h,0BBh,0F1h,0C9h,0F5h,03Ah,0EFh,03Fh,018h,04h,0F5h
	db	0DDh,07Eh,0FDh,032h,0F3h,03Fh,0F1h,0C3h,04Fh,032h,0FEh,06h,0D2h,01Dh,035h,0FEh
	db	02h,030h,06h,0F5h,0AFh,032h,0F3h,03Fh,0F1h,0C3h,0D8h,034h,0D8h,03Fh,018h,02Ah
	db	0DDh,0E5h,0E1h,011h,0Ah,00h,019h,07Eh,0FEh,01h,028h,01h,0AFh,021h,0EEh,032h
	db	077h,023h,0AFh,03Dh,0C9h,0F5h,0DDh,07Eh,019h,0FEh,0FFh,011h,02Eh,00h,020h,03h
	db	011h,012h,00h,0F1h,0C9h,0E1h,0AFh,0C9h,015h,03Eh,03h,0CDh,0D4h,034h,0F5h,03Ah
	db	0F0h,03Fh,032h,0F1h,03Fh,0F1h,0D2h,0BBh,0C9h,0F5h,03Ah,0F1h,03Fh,032h,0F0h,03Fh
	db	0F1h,021h,0D2h,0BBh,0C9h,070h,023h,03Ah,0F0h,03Fh,0DDh,077h,032h,0C9h,0B7h,0EDh
	db	052h,0C0h,03Ah,0F0h,03Fh,0DDh,096h,032h,0C9h,0F5h,03Ah,0F0h,03Fh,032h,0F2h,03Fh
	db	0F1h,021h,0DEh,0BBh,0C9h,0F5h,03Ah,0F2h,03Fh,032h,0F0h,03Fh,0F1h,021h,0C6h,0BBh
	db	0C9h,0F5h,03Ah,0F0h,03Fh,0DDh,077h,038h,0F1h,0EBh,0EDh,0B0h,0C9h,0F5h,0DDh,07Eh
	db	038h,032h,0F0h,03Fh,0F1h,0DDh,04Eh,019h,0C9h,0EBh,02Bh,02Bh,0B7h,0C8h,036h,00h
	db	02Bh,036h,00h,01h,011h,00h,09h,077h,02Bh,0C9h,00h,0CDh,024h,03Fh,0C3h,066h
	db	030h,0CDh,0BBh,03Fh,00h,0CDh,078h,0Dh,0C3h,06Fh,031h,0CDh,055h,030h,0C3h,07Bh
	db	0Dh,0CDh,0C7h,033h,0C3h,088h,0Dh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh
	db	0FFh,0FFh,0FFh,0FFh,01h,09h,00h,09h,04Eh,023h,066h,069h,019h,030h,01h,03Ch
	db	0EBh,032h,0EEh,03Fh,0C1h,0E1h,0C9h,030h,01h,014h,034h,035h,0C0h,014h,0C9h,0DDh
	db	07Eh,0FCh,0B7h,020h,01h,014h,083h,0C9h,030h,01h,014h,01Ch,01Dh,0C0h,014h,0C9h
	db	011h,02h,00h,0CDh,09h,0Eh,0C2h,07h,0Ch,03Ah,0EDh,03Fh,0B7h,0CAh,07h,0Ch
	db	0F3h,0DDh,07Eh,06h,032h,0A4h,03Fh,0DDh,07Eh,00h,032h,0B4h,03Fh,0DDh,06Eh,0Ch
	db	0DDh,066h,0Dh,022h,09Fh,03Fh,0E1h,0FDh,0E5h,0DDh,0E5h,0E5h,01h,00h,00h,03Eh
	db	0C9h,032h,038h,00h,03Ah,0EDh,03Fh,0D3h,0FEh,0CDh,099h,03Fh,0E5h,029h,07Ch,0E6h
	db	01h,067h,0CBh,0FCh,0D1h,07Eh,023h,0B6h,023h,020h,01h,03h,0E3h,0EDh,052h,019h
	db	0E3h,013h,028h,06h,0CBh,074h,028h,0EDh,018h,0DFh,03Ah,0CFh,0F2h,0D3h,0FEh,03Eh
	db	0C3h,032h,038h,00h,0E1h,0DDh,0E1h,0FDh,0E1h,0E5h,0C3h,01Bh,0Ch,0C5h,0D5h,05Ah
	db	016h,00h,021h,01h,00h,019h,0EBh,03Eh,00h,06h,020h,0Eh,00h,021h,00h,080h
	db	0B7h,0DDh,021h,010h,040h,0FDh,021h,00h,00h,0CDh,01Ch,00h,0E1h,0C1h,0C9h,0DDh
	db	07Eh,015h,077h,0E5h,0DDh,0E5h,0E1h,01h,00h,02h,03Eh,046h,0EDh,0B1h,028h,03h
	db	0B7h,018h,014h,0E5h,0C5h,06h,04h,011h,0E9h,03Fh,01Ah,0AEh,020h,05h,023h,013h
	db	010h,0F8h,037h,0C1h,0E1h,030h,0E3h,0E1h,07Eh,0D0h,0CBh,0BEh,0C9h,041h,054h,031h
	db	036h,01Dh,00h,00h,00h,00h,00h,00h,00h,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh,0FFh
	db	0FFh,0FFh,0FFh,0FFh

; FAT16 patch table starts here
PATCHT:
	; (removed _FORMAT patching)
;	dw	0370h
;	db	03h
	dw	0372h
	db	01h
	dw	0c04h
	db	02h
	dw	0c0bh
	db	01h
	dw	0c23h
	db	01h
	dw	0cbah
	db	02h
	dw	0cbeh
	db	00h
	dw	0cc6h
	db	02h
	dw	0d39h
	db	032h
	dw	0d6dh
	db	08ch
	dw	0dfbh
	db	018h
	dw	01534h
	db	02h
	dw	0158bh
	db	05h
	dw	0198bh
	db	02h
	dw	01a0bh
	db	02h
	dw	01b35h
	db	01h
	dw	01b87h
	db	02h
	dw	01bedh
	db	02h
	dw	01c05h
	db	02h
	dw	01c0dh
	db	05h
	dw	01c53h
	db	05h
	dw	01ca5h
	db	03h
	dw	01cb1h
	db	05h
	dw	01cbdh
	db	01h
	dw	01cc5h
	db	01h
	dw	01cffh
	db	01h
	dw	021a4h
	db	00h
	dw	0229ch
	db	02h
	dw	02322h
	db	02h
	dw	02436h
	db	02h
	dw	02476h
	db	02h
	dw	02589h
	db	01h
	dw	025cdh
	db	02h
	dw	025efh
	db	02h
	dw	0262dh
	db	02h
	dw	02649h
	db	02h
	dw	026c2h
	db	01h
	dw	028ddh
	db	01h
	dw	02981h
	db	01h
	dw	02987h
	db	01h
	dw	02a4bh
	db	09h
	dw	02b5eh
	db	03h
	dw	02bbdh
	db	04h
	dw	02bf9h
	db	01h
	dw	02c33h
	db	02h
	dw	02d33h
	db	02h
	dw	02d55h
	db	01h
	dw	02d65h
	db	02h
	dw	02dc0h
	db	010h
	dw	02dd7h
	db	02h
	dw	02e27h
	db	01h
	dw	02e64h
	db	01h
	dw	02e81h
	db	01h
	dw	02eb5h
	db	01h
	dw	02ed8h
	db	01h
	dw	02eddh
	db	01h
	dw	02f03h
	db	01h
	dw	02f0bh
	db	02h
	dw	02f65h
	db	01h
	dw	02f6fh
	db	02h
	dw	02f9bh
	db	05h
	dw	02fbeh
	db	02h
	dw	02fe0h
	db	01h
	dw	02ffeh
	db	01h
	dw	0300dh
	db	02h
	dw	03031h
	db	029h
	dw	0305dh
	db	0f8h
	dw	03156h
	db	02ah
	dw	03253h
	db	00h
	dw	032dah
	db	00h
	dw	03318h
	db	026h
	dw	03375h
	db	08h
	dw	03382h
	db	07h
	dw	0338bh
	db	0eh
	dw	0339bh
	db	054h
	dw	03437h
	db	00h
	dw	03449h
	db	02h
	dw	03464h
	db	02h
	dw	03483h
	db	03h
	dw	034cah
	db	02h
	dw	034d4h
	db	02h
	dw	03538h
	db	02h
	dw	039cbh
	db	02h
	dw	03a82h
	db	02h
	dw	03cd4h
	db	02h
	dw	03ef3h
	db	0f8h
	dw	03fech
	db	013h
	dw	0ffffh

; end of file
; end of file
; end of file
; end of file
;
;
;
;
;
;
;
;

