;********************************************************************************
;*																				*
;*								License and T&C									*
;*																				*
;*			GR8BIT storage subsystem microcontroller microcode version 3.0		*
;*			     This software is copyleft 2011-12 by Eugeny Brychkov			*
;*																				*
;* You can use and modify its source and executable, given two conditions are	*
;* met: (a) you keep this Copyleft statement in the source code, and you do		*
;* not distribute software and resulting executables on the commercial basis	*
;* and for the purposes, not related to GR8BIT platform, without written		*
;* permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)					*
;*																				*
;********************************************************************************
;*			File name: GR8STORM_DATA.INC										*
;********************************************************************************

; This file represents data (strings, tables, images of sectors)
GR8STORM_DATA	CODE_PACK


; ***********************************************************************************************************
; *									Strings for GR8BIT subsystem initialization								*
; ***********************************************************************************************************
DeviceID:		db	"PIC18F46K22 v.3.0",0xd,0xa,0
ModeID:			db	0xd,0xa,0xa,"ModeID: 0x",0
MessageStr:		db	"]",0
FloppyStr1:		db	0x0d,0x0a,0x0a,"Floppy disk subsystem:",0x0d,0x0a,0
FloppyStr0:		db	"Disabled",0x0d,0x0a,0
FloppyStr2:		db	"Total: ",0
FloppyStr3:		db	" (phy), ",0
FloppyStr4:		db	" (log)",0
HDDStr1:		db	0x0d,0x0a,0x0a,"Hard disk subsystem:",0x0d,0x0a,0
HDDStr2:		db	0x0d,0x0a,"  (",0
HDDStr3:		db	")",0x0d,0x0a,0
ErrorStr:		db	"(error)",0xd,0xa,0
HDDStr4:		db	" (drives), ",0
HDDStr5:		db	" (LVOLs)",0xd,0xa,0
StrNotInstalled:db		"Not installed",0x0d,0x0a,0
StrPresent:		db		"Present",0x0d,0x0a,0

; ***********************************************************************************************************
; *											HDD partition type table										*
; ***********************************************************************************************************
; source: http://www.win.tue.nl/~aeb/partitions/partition_types-1.html
HTyTbl:
	db	0x01,LOW LFAT12,HIGH LFAT12		;,UPPER LFAT12
	db	0x04,LOW LFAT16,HIGH LFAT16		;,UPPER LFAT16
	db	0x05,LOW LEXPRT,HIGH LEXPRT		;,UPPER LEXPRT
	db	0x06,LOW LFAT16,HIGH LFAT16		;,UPPER LFAT16
	db	0x07,LOW LNTFS,HIGH LNTFS		;,UPPER LNTFS
	db	0x08,LOW LAIXBT,HIGH LAIXBT		;,UPPER LAIXBT
	db	0x09,LOW LAIXDT,HIGH LAIXDT		;,UPPER LAIXDT
	db	0x51,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x63,LOW LUNIXV,HIGH LUNIXV		;,UPPER LUNIXV
	db	0x64,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x65,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x66,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x67,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x68,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x69,LOW LNETWR,HIGH LNETWR		;,UPPER LNETWR
	db	0x82,LOW LSOLAR,HIGH LSOLAR		;,UPPER LSOLAR
	db	0x83,LOW LLINUX,HIGH LLINUX		;,UPPER LLINUX
	db	0x9f,LOW LBSDOS,HIGH LBSDOS		;,UPPER LBSDOS
	db	0xa5,LOW LFRBSD,HIGH LFRBSD		;,UPPER LFRBSD
	db	0xa6,LOW LOPBSD,HIGH LOPBSD		;,UPPER LOPBSD
	db	0xa8,LOW LMACOS,HIGH LMACOS		;,UPPER LMACOS
	db	0xa9,LOW LNTBSD,HIGH LNTBSD		;,UPPER LNTBSD
	db	0xdb,LOW LCPM,HIGH LCPM			;,UPPER LCPM
	db	0xfb,LOW LVMWRE,HIGH LVMWRE		;,UPPER LVMWRE
	db	0x00,LOW LOTHER,HIGH LOTHER		;,UPPER LOTHER

LFAT12:	db	"FAT12",0
LFAT16:	db	"FAT16",0
LEXPRT:	db	"EXTND",0
LNTFS:	db	"NTFS ",0
LAIXBT:	db	"AIXBT",0
LAIXDT:	db	"AIXDT",0
LUNIXV:	db	"UNIXV",0
LNETWR:	db	"NTWRE",0
LSOLAR:	db	"SOLAR",0
LLINUX:	db	"LINUX",0
LBSDOS:	db	"BSDOS",0
LFRBSD:	db	"FRBSD",0
LOPBSD:	db	"OPBSD",0
LMACOS:	db	"MACOS",0
LNTBSD:	db	"NTBSD",0
LCPM:	db	"CP/M ",0
LVMWRE:	db	"VMWRE",0
LOTHER:	db	"OTHER",0
LEMPTY:	db	"EMPTY",0


; ***********************************************************************************************************
; *								Pointers to the subroutines to call in GR8STORM								*
; ***********************************************************************************************************
STORM_Pointers:
	dw	STORM_TopMenu-STORM_Pointers
	dw	STORM_OpModeMenu-STORM_Pointers
	dw	STORM_MessageMenu-STORM_Pointers
	dw	STORM_Disabled-STORM_Pointers
	dw	STORM_Enabled-STORM_Pointers
	dw	STORM_FloppyFirst-STORM_Pointers
	dw	STORM_HDDFirst-STORM_Pointers
	dw	STORM_Standard-STORM_Pointers
	dw	STORM_Compatible-STORM_Pointers


; ***********************************************************************************************************
; *								Call table on the identifier in the output string							*
; ***********************************************************************************************************
CallTable:
	dw	FloppyDisableEnable			; 0x80 (control character code in output sequence)
	dw	HDDDisableEnable			; 0x81
	dw	DeviceOrderStr				; 0x82
	dw	FDDModeDisplay				; 0x83
	dw	PhantomModeDisplay			; 0x84
	dw	DisplayMessage				; 0x85
	dw	DisplaySpT					; 0x86
	dw	DisplayHeads				; 0x87
	dw	DisplaySecFat				; 0x88
	dw	DisplayMaxCluster			; 0x89
	dw	DisplaySecClus				; 0x8a
	dw	DisplaySize					; 0x8b
	dw	DisplaySecFATMax			; 0x8c
	dw	DisplaySecClusMax			; 0x8d
	dw	DisplayPartitionNum			; 0x8e
	dw	DisplayHDDDRVPRMNum			; 0x8f
	dw	DisplayMBRVolumeEntry		; 0x90
	dw	DisplayFSType				; 0x91
	dw	DisplayDirEnt				; 0x92


; ***********************************************************************************************************
; *									Call tables to call on the user input									*
; ***********************************************************************************************************
CallTableL1:
	dw	ReturnCarryFlag				; "0" (exit to higher level menu)
	dw	OpModeMenuSelect			; "1"
	dw	MessageMenuSelect			; "2"
	dw	MediaConfigMenuSelect		; "3"
	dw	DiagnosticOutputSelect		; "4"

CallTableL2OpMode:
	dw	ReturnCarryFlag				; "0" (exit to higher level menu)
	dw	FloppyEnaDisa				; "1"
	dw	HDDEnaDisa					; "2"
	dw	DeviceOrderSet				; "3"
	dw	FDDModeSet					; "4"
	dw	PhantomModeSet				; "5"
	dw	WriteOpModeEEPROM			; "6"

CallTableL2MediaConfig:
	dw	ReturnCarryFlag				; "0" (exit to higher level menu)
	dw	Enumerate					; "1"
	dw	WriteDOS2BootSelect			; "2"
	dw	PartitionHDDSelect			; "3"
	dw	ChangePartitionActivation	; "4"
	dw	InitLVOL					; "5"

CallTableL2MessageMenu:
	dw	ReturnCarryFlag				; "0" (exit to higher level menu)
	dw	ClearMessageSet				; "1"
	dw	NewMessageSet				; "2"

CallTableTunePartition:
	dw	SetSecFAT					; "1"
	dw	SetSecClus					; "2"

CallTableL3DiagOutputs:
	dw	ReturnCarryFlag				; "0" (exit to higher level menu)
	dw	DisplayMCWorkset			; "1"
	dw	DisplayMCBuffer				; "2"
	dw	DisplayFDD0Boot				; "3"
	dw	DisplayHDD0Struct			; "4"
	dw	DisplayHDD1Struct			; "5"

CallTableLVOLInit:
	dw	ReturnCarryFlag				; "0" (exit to higher level menu)
	dw	RecoverLVOLBootAndFAT		; "1"
	dw	ReInitializeLVOL			; "2"


; ***********************************************************************************************************
; *											GR8STORM dialogue strings										*
; ***********************************************************************************************************
STORM_InitStr:			db	0xd,0xa,0xa,"GR8BIT Storage Manager v.3.0",0
STORM_TopMenu:			db	0x0d,0x0a,0x0a
						db	"GR8STORM options:",0xd,0xa
						db	"1: Operation mode",0x9,"2: Boot message",0x9,0x9,"3: Media configuration",0xd,0xa
						db	"4: Diagnostic outputs",0x9,"0: Exit",0xd,0xa,"Choice>",0
STORM_OpModeMenu:		db	0x0d,0x0a,0xa
						db	"1: Floppy subsystem - ",0x80,0xd,0xa,"2: HDDs subsystem - ",0x81,0xd,0xa
						db	"3: Device order - ",0x82,0xd,0xa,"4: FDD mode - ",0x83,0x0d,0x0a
						db	"5: Phantom floppy - ",0x84,0x0d,0x0a,"6: Write settings to EEPROM",0xd,0xa
						db	"0: Exit menu",0xd,0xa,"Choice>",0
STORM_MessageMenu:		db	0xd,0xa,0xa
						db	"Current message is {",0x85,"}",0x0d,0x0a
						db	"1: Clear message",0x9,0x9,"2: Set new message (24 chars max)",0x0d,0x0a
						db	"0: Exit menu",0xd,0xa,"Choice>",0
STORM_MediaConfigMenu:	db	0xd,0xa,0xa
						db	"1: Reset and enumerate hardware",0x9,"2: Update Floppy boot sector to DOS2",0xd,0xa
						db	"3: Partition hard disk",0x9,0x9,"4: Change partition activation status",0xd,0xa
						db	"5: Initialize/Recover Logical Volume (LVOL)",0x9,"0: Exit menu",0xd,0xa
						db	"Choice>",0
STORM_DiagnosticMenu:	db	0xd,0xa,0xa,"1: Display MC's workset",0x9,0x9,"2: Display MC's buffer contents",0xd,0xa
						db	"3: Display FDD0's floppy disk's boot sector",0xd,0xa,"4: Display HDD0's MBR and LVOLs' boot sectors",0xd,0xa
						db	"5: Display HDD1's MBR and LVOLs' boot sectors",0xd,0xa,"0: Exit menu",0xd,0xa,"Choice>",0
STORM_MemoryDump:		db	0xd,0xa,0xa,"MC memory dump:",0xd,0xa,0
STORM_BufferDump:		db	0xd,0xa,0xa,"MC buffer dump (CTRL-STOP to exit):",0xd,0xa,0
STORM_PressContinue:	db	0xd,0xa,"Press any key to continue (CTRL-STOP to break)",0
STORM_NoFDD0Present:	db	0xd,0xa,"Floppy drive 0 is not present",0
STORM_NoHDD0Present:	db	0xd,0xa,"Hard drive 0 is not present",0
STORM_NoHDD1Present:	db	0xd,0xa,"Hard drive 1 is not present",0
STORM_FDD0BootStr:		db	0xd,0xa,"Boot sector of floppy disk in floppy drive 0:",0
STORM_NoHDDPresent:		db	0xd,0xa,"Hard drive ",0x8f," is not present",0
STORM_HDDMBR:			db	0xd,0xa,"Hard drive ",0x8f," master boot record (MBR)",0
STORM_HDDBootSector:	db	0xd,0xa,"Hard drive ",0x8f," volume entry ",0x90," boot record",0
STORM_FDD0MotorDidNotStop:	db	0xd,0xa,"FDD0 motor did not stop within specified period!",0xd,0xa,0
STORM_Enumerate:		db	0xd,0xa,"------------------------------",0xd,0xa,0
STORM_Zero:				db	"0: ",0
STORM_CR:				db	0x0d,0x0a,0
STORM_NewMsgPrompt:		db	0xd,0xa,"New message: ",0x2,0x18,0
STORM_FloppyDriveInsert:	db	0xd,0xa,"Insert floppy disk into physical drive 0 (A:) "
						db	"and press ENTER key",0xd,0xa,0x1,0
STORM_Aborted:			db	0xd,0xa,"Aborted",0xd,0xa,0
STORM_Floppy0Missing:	db	0xd,0xa,"Floppy drive 0 is missing",0xd,0xa,0
STORM_SectorReadError:	db	0xd,0xa,"Sector read error",0xd,0xa,0
STORM_SectorWriteError:	db	0xd,0xa,"Sector write error",0xd,0xa,0
STORM_NoHDDInstalled:	db	0xd,0xa,"No HDDs installed",0xd,0xa,0
STORM_HDDList:			db	0xd,0xa,0x8f,": Hard disk drive ",0x8f,0
STORM_HDDPrompt:		db	0xd,0xa,"?>",0
STORM_PartitionList:	db	0xd,0xa,"# S Sys   StartLBA CountLBA",0xd,0xa,0
STORM_RepartitionDiskQ:	db	0xd,0xa,"Repartition the disk?",0xd,0xa,"All current data will be lost! (upcase Y to accept)>",0x01,0
STORM_MBR1:				db	0xeb,0xfe,0x90,"GR8BIT PARTITION MANAGER 3.0",0
STORM_DiskSizeIs:		db	0xd,0xa,"Disk size is ",0
STORM_InvalidInput:		db	0xd,0xa,"Invalid input",0xd,0xa,0
STORM_MBR2:				db	"GR8B",0
STORM_MBR3:				db	0x55,0xaa,0
STORM_WhichPartitionWanted:	db	0xd,0xa,"Please select type:",0xd,0xa,"1: FAT12",0xd,0xa,"2: FAT16",0xd,0xa,">",0
STORM_MenuPartitionParameters:	db	0xd,0xa,0xa,"* Fixed parameters:",0x9,"File system type: FAT1",0x91,0xd,0xa
						db	"Sector size = 200h",0x9,"Reserved sectors = 1",0xd,0xa
						db	"Media ID = F0h",0x9,0x9,"Directory entries = ",0x92,"h",0xd,0xa
						db	"Number of FATs = 2",0x9,"Sectors per track = ",0x86,0xd,0xa
						db	"Heads = ",0x87,0x9,0x9,"Hidden sectors = ",0x86,0xd,0xa
						db	"* Configurable parameters:",0xd,0xa
						db	"1: Sectors per FAT copy = ",0x88,"h [MaxClus = ",0x89,"]",0xd,0xa
						db	"2: Sectors per cluster = ",0x8a,"h -> Size = ",0x8b,0xd,0xa
						db	"3: Accept and next entry",0xd,0xa
						db	"4: Accept and create",0xd,0xa,"Choice>",0
STORM_RecoverLVOL:		db	0xd,0xa,"LVOL initialization/recovery information:",0xd,0xa
						db	"File system type = FAT1",0x91,0xd,0xa
						db	"Sectors per FAT copy = ",0x88,"h [MaxClus = ",0x89,"]",0xd,0xa
						db	"Sectors per cluster = ",0x8a,"h -> Size = ",0x8b,0xd,0xa
						db	"* Available options:",0xd,0xa
						db	"1: Recover LVOL's boot record and signature of first FAT sector",0xd,0xa
						db	"2: Re-initialize the LVOL structures (quick format)",0xd,0xa
						db	"0: Exit menu",0xd,0xa,"Choice>",0
STORM_CreatingLVOLs:	db	0xd,0xa,"Creating disk structures...",0
STORM_PartitionExceeds:	db	0xd,0xa,"Partition exceeds HDD size",0x0d,0x0a,0
STORM_PartitionNumber:	db	0xd,0xa,"Partition #",0x8e,0
STORM_AreYouSure:		db	0xd,0xa,"Are you sure?",0xd,0xa,"All current data will be lost! (upcase Y to accept)>",0x01,0
STORM_OptionSecFAT:		db	0xd,0xa,"Sectors per FAT copy (decimal, 3..",0x8c,")> ",0
STORM_OptionSecClus:	db	0xd,0xa,"Sectors per Cluster (decimal, 2^(0..",0x8d,"))> ",0
STORM_Disabled:			db	"Disabled",0
STORM_Enabled:			db	"Enabled",0
STORM_FloppyFirst:		db	"Floppy first",0
STORM_HDDFirst:			db	"HDD first",0
STORM_Standard:			db	"Standard",0
STORM_Compatible:		db	"Compatible",0
STORM_RebootSystem:		db	0xd,0xa,0xa,"Vital changes to storage subsystem configuration were made.",0xd,0xa
						db	"Please reboot the system for changes to take effect.",0xd,0xa
						db	"Using system without reboot may cause system instability or",0xd,0xa
						db	"damage to the data of the affected volumes",0xd,0xa,0
STORM_LVOL_Act_Deact:	db	0xd,0xa,"Please enter number of volume to toggle",0xd,0xa
						db	"(toggling empty volume will have no effect)",0xd,0xa
						db	"Choice>",0
STORM_LVOL_Initialize:	db	0xd,0xa,"Please enter number of the volume to initialize/recover",0xd,0xa
						db	"Choice>",0
STORM_LVOLEmpty:		db	0xd,0xa,"This logical volume is not defined",0
STORM_LVOL_AD_Confirm:	db	0xd,0xa,"Confirm update of MBR (updace Y)>",0x01,0
STORM_StringsEnd:


; ***********************************************************************************************************
; *								Boot sector structures for floppies and LVOLs								*
; ***********************************************************************************************************
; DOS2 _diskette_ boot sector (with "VOL_ID" tag) - can boot in DOS2 mode
BootSectDOS2:
	db		0xeb,0xfe,0x90,0x47,0x52,0x38,0x42,0x49,0x54,0x33,0x30,0x00,0x02,0x00,0x01,0x00
		;	  EB   FE   90  'G'  'R'  '8'  'B'  'I'  'T'  '3'  '0' BytesPSec  SpC  ResvSect
	db		0x02,0x00,0x00,0x40,0x0b,0xf0,0x00,0x00,0x12,0x00,0x02,0x00,0x00,0x00,0x18,0x10
		;	nFAT RootDirEn TotalSect  MID SecPerFAT SecPerTra  NumHeads  HidSect
	db		0x56,0x4f,0x4c,0x5f,0x49,0x44,0x00,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x00,0x00
		;	 'V'  'O'  'L'  '_'  'I'  'D' Undl  DiskSerialNumber
	db		0xd0,0xed,0x53,0x6a,0xc0,0x32,0x72,0xc0,0x36,0x67,0x23,0x36,0xc0,0x31,0x1f,0xf5
	db		0x11,0xab,0xc0,0x0e,0x0f,0xcd,0x7d,0xf3,0x3c,0x28,0x26,0x11,0x00,0x01,0x0e,0x1a
	db		0xcd,0x7d,0xf3,0x21,0x01,0x00,0x22,0xb9,0xc0,0x21,0x00,0x3f,0x11,0xab,0xc0,0x0e
	db		0x27,0xcd,0x7d,0xf3,0xc3,0x00,0x01,0x69,0xc0,0xcd,0x00,0x00,0x79,0xe6,0xfe,0xd6
	db		0x02,0xf6,0x00,0xca,0x22,0x40,0x11,0x85,0xc0,0x0e,0x09,0xcd,0x7d,0xf3,0x0e,0x07
	db		0xcd,0x7d,0xf3,0x18,0xb8,0x42,0x6f,0x6f,0x74,0x20,0x65,0x72,0x72,0x6f,0x72,0x0d
	db		0x0a,0x50,0x72,0x65,0x73,0x73,0x20,0x61,0x6e,0x79,0x20,0x6b,0x65,0x79,0x20,0x66
	db		0x6f,0x72,0x20,0x72,0x65,0x74,0x72,0x79,0x0d,0x0a,0x24,0x00,0x4d,0x53,0x58,0x44
	db		0x4f,0x53,0x20,0x20,0x53,0x59,0x53,0x00,0x00,0x00,0x00
EndOfBootSectDOS2:

; Boot code of the floppy disk's boot sector (asks to insert another disk to boot)
;	C030	ret		nc						d0				; return first time when CY=0
;	C031	ld		(ErrH_Routine+1),de		ed 53 6a c0		; set up disk ROM select calling address
;	C035	ld		(CheckWarmBoot+1),a		32 72 c0		; check warm boot flag - keep retrying booting on warm boot
;	C038	ld		(hl),LOW Err_Handler_ptr	36 67
;	C03A	inc		hl						23
;	C03B	ld		(hl),HIGH Err_Handler_ptr	36 c0		; set up custom error handler routine
;BootAttempt:
;	C03D	ld		sp,0f51fh				31 1f f5		; reset stack to temp area
;	C040	ld		de,MSXDOS_FCB			11 ab c0
;	C043	ld		c,0fh					0e 0f
;	C045	call	0f37dh					cd 7d f3		; try opening file
;	C048	inc		a						3c
;	C049	jr		z,CheckWarmBoot			28 26			; error - go checking warm boot and try/go to BASIC
;	C04B	ld		de,0100h				11 00 01
;	C04E	ld		c,1ah					0e 1a
;	C050	call	0f37dh					cd 7d f3		; set data transfer address
;	C053	ld		hl,0001h				21 01 00
;	C056	ld		(Record_Size),hl		22 b9 c0		; set MSXDOS.SYS record size to 1 byte
;	C059	ld		hl,3f00h				21 00 3f
;	C05C	ld		de,MSXDOS_FCB			11 ab c0
;	C05F	ld		c,27h					0e 27
;	C061	call	0f37dh					cd 7d f3		; read max 3f00h bytes from MSXDOS.SYS
;	C064	jp		0100h					c3 00 01		; and execute it (or go ro error handler if error)
;Err_Handler_ptr:
;	C067	dw		ErrH_Routine			69 c0			; pointer to code of error handler
;ErrH_Routine:
;	C069	call	0000h					cd 00 00		; call to set disk ROM image into bank 1
;	C06C	ld		a,c        				79
;	C06D	and		0feh					e6 fe
;	C06F	sub		02h						d6 02			; error = 2?
;CheckWarmBoot:
;	C071	or		0h						f6 00			; and not a warm boot?
;	C073	jp		z,4022h					ca 22 40		; then go to BASIC
;	C076	ld		de,BootErrStr			11 85 c0
;	C079	ld		c,09h					0e 09
;	C07B	call	0f37dh					cd 7d f3		; otherwise display "Boot error..." message
;	C07E	ld		c,07h					0e 07
;	C080	call	0f37dh					cd 7d f3		; and wait for keystroke
;	C083	jr		BootAttempt				18 b8			; and loop back resetting stack back (no file close required)
;BootErrStr:
;	C085	db		"Boot Error",0dh,0ah
;			db		"Press any key for retry",0dh,0ah
;			db		'$'
;MSXDOS_FCB:
;	C0AB	db		0						00				; drive number to load from (0=A:)
;	C0AC	db		"MSXDOS  SYS"			4d 53 58 44 4f 53 20 20 53 59 53	; file name (11 bytes)
;	C0B7	db		0,0
;Record_Size:
;	C0B9	dw		0

; _LVOL_ boot sector (without "Boot error - press any key for retry") - can boot in DOS2 mode
BootSectLVOL:
	db		0xeb,0xfe,0x90,0x47,0x52,0x38,0x42,0x49,0x54,0x33,0x30,0x00,0x02,0x40,0x01,0x00
		;	  EB   FE   90  'G'  'R'  '8'  'B'  'I'  'T'  '3'  '0' BytesPSec  SpC  ResvSect
	db		0x02,0x00,0x02,0x00,0x00,0xf0,0x00,0x01,0x20,0x00,0x10,0x00,0x00,0x00,0x18,0x20
		;	nFAT RootDirEn TotalSect  MID SecPerFAT SecPerTra  NumHeads  HidSect   (Jump)
 	db		0x56,0x4f,0x4c,0x5f,0x49,0x44,0x00,0xff,0xff,0xff,0xff,0x46,0x41,0x54,0x31,0x36
		;	  "VOL_ID"/BigSize  VOLID_rem Undl  Volume serial num	'F'  'A'  'T'  '1'  '6'
 	db		0x47,0x52,0x38,0x42,0x49,0x54,0x5f,0x50,0x41,0x52,0x54,0x4d,0x47,0x52,0x33,0x30
		;	 'G'  'R'  '8'  'B'  'I'  'T'  '_'  'P'  'A'  'R'  'T'  'M'  'G'  'R'  '3'  '0'
	db		0xd0,0xed,0x53,0x7d,0xc0,0x11,0x7a,0xc0,0x73,0x23,0x72,0x11,0x82,0xc0,0x0e,0x0f
	db		0xcd,0x7d,0xf3,0x3c,0xca,0x22,0x40,0x11,0x00,0x01,0x0e,0x1a,0xcd,0x7d,0xf3,0x21
	db		0x01,0x00,0x22,0x90,0xc0,0x21,0x00,0x3f,0x11,0x82,0xc0,0x0e,0x27,0xd5,0xcd,0x7d
	db		0xf3,0xd1,0x0e,0x10,0xcd,0x7d,0xf3,0xc3,0x00,0x01,0x7c,0xc0,0xcd,0x00,0x00,0xc3
	db		0x22,0x40,0x00,0x4d,0x53,0x58,0x44,0x4f,0x53,0x20,0x20,0x53,0x59,0x53,0x00,0x00
		;			        'M'  'S'  'X'  'D'  'O'  'S'  ' '  ' '  'S'  'Y'  'S'
	db		0x00,0x00
		;    RecSize
BootSectLVOLEnd:

; Boot code of the hard drive boot sector (does not ask to insert another disk to boot)
;	C040	ret		nc						d0				; return first time when CY=0
;	CO41	ld		(ErrH_Routine+1),de		ed 53 7d c0		; set disk ROM slot enable routine call in the disk error handler routine
;	C045	ld		de,Err_Handler_ptr		11 7a c0
;	C048	ld		(hl),e					73
;	C049	inc		hl						23
;	C04A	ld		(hl),d					72				; set disk error handler routine pointer in the F323 to the pointer to it
;	C04B	ld		de,MSXDOS_FCB			11 82 c0
;	C04E	ld		c,0fh					0e 0f
;	C050	call	0f37dh					cd 7d f3		; open MSXDOS.SYS FCB/look for file
;	C053	inc 	a						3c
;	C054	jp 		z,04022h				ca 22 40		; file was not found, invoke BASIC immediately
;	C057	ld		de,0100h				11 00 01
;	C05A	ld		c,01ah					0e 1a
;	C05C	call	0f37dh					cd 7d f3		; set data transfer address to 0x100
;	C05F	ld		hl,0001					21 01 00
;	C062	ld		(Record_Size),hl		22 90 c0		; set FCB field +0eh (record size) to 1 byte
;	C065	ld		hl,03f00h				21 00 3f		; max 3f00h (16128) bytes to read
;	C068	ld		de,MSXDOS_FCB			11 82 c0
;	C06B	ld		c,27h					0e 27
;	C06D	push	de						d5				; preserve FCB address for following close
;	C06E	call	0f37dh					cd 7d f3		; read MSXDOS.SYS; it will read max 16128 bytes; on error handler will be called
;	C071	pop		de						d1				; restore FDC address
;	C072	ld		c,10h					0e 10
;	C074	call	0f37dh					cd 7df3			; close MSXDOS.SYS file
;	C077	jp		0100h					c3 00 01		; run it
;Err_Handler_ptr:
;	C07A	dw		ErrH_Routine			7c c0			; pointer to the custom error handler
;ErrH_Routine:
;	C07C	call	0000h					cd 00 00		; calls disk ROM slot enable routine (set dynamically previosly)
;	C07F	jp		4022h					c3 22 40		; calls disk ROM hook which invokes disk BASIC
;MSXDOS_FCB:
;	C082	db		0						00				; drive number to load from (0=A:)
;	C083	db		"MSXDOS  SYS"			4d 53 58 44 4f 53 20 20 53 59 53	; file name (11 bytes)
;	C08E	db		0,0
;Record_Size:
;	C090	dw		0


; ***********************************************************************************************************
; *							Default floppy disk DPB images: Standard and Compatible							*
; ***********************************************************************************************************
StdDPB:		; standard 1.44MiB disk DPB
		;	MEDIA         DIRMSK    CLUSMSK      FIRFAT      MAXENT           MAXCLUS        FIRDIR
		db	0xf0,0x00,0x02,0x0f,0x04,0x00,0x01,0x01,0x00,0x02,0xe0,0x21,0x00,0x20,0x0b,0x09,0x13,0x00
		;         SECSIZ       DIRSHFT  CLUSSHFT        FATCNT      FIRREC            FATSIZ
CompDPB:
		;	MEDIA         DIRMSK    CLUSMSK      FIRFAT      MAXENT           MAXCLUS        FIRDIR
		db	0xf0,0x00,0x02,0x0f,0x04,0x03,0x03,0x01,0x00,0x02,0x70,0x0e,0x00,0xcd,0x02,0x03,0x07,0x00
		;         SECSIZ       DIRSHFT  CLUSSHFT        FATCNT      FIRREC            FATSIZ


;********************************************************************************
;*			END OF: GR8STORM_DATA.INC											*
;********************************************************************************