;********************************************************************************
;*																				*
;*								License and T&C									*
;*																				*
;*			GR8BIT storage subsystem microcontroller microcode version 3.0		*
;*			     This software is copyleft 2011-12 by Eugeny Brychkov			*
;*																				*
;* You can use and modify its source and executable, given two conditions are	*
;* met: (a) you keep this Copyleft statement in the source code, and you do		*
;* not distribute software and resulting executables on the commercial basis	*
;* and for the purposes, not related to GR8BIT platform, without written		*
;* permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)					*
;*																				*
;********************************************************************************
;*			File name: GR8STORM_DEFINITIONS.INC									*
;********************************************************************************


; ***********************************************************************************************************
; *										 Definitions of the constants										*
; ***********************************************************************************************************
RETRIES			equ		0x04		; retries for floppy drive read/write routine

; GAP lengths can be found in SMC37C65 data sheet
; Info on DSPEC values can be found here http://wiki.osdev.org/Floppy_Disk_Controller, also http://www.isdaman.com/alsos/hardware/fdc/floppy.htm
; and also http://www.mathematik.uni-kl.de/~hannes/en/floppy.diff

; 1.44MB diskette / speed configuration (500Mbit/S)
GPL1_144		equ		0x1B		; gap length when reading
GPL2_144		equ		0x54		; gap length when formatting
DSPEC1_144		equ		0xAF		; A=step rate 6ms, F=head unload delay (240ms)
DSPEC2_144		equ		0x11		; 8=16ms head load time, 1=no DMA
DSPD_144		equ		0x0			; speed 500kB/s
SpT_144			equ		0x12		; sectors per track (18)

; 720K diskette / speed configuration (250Mbit/s)
GPL1_720		equ		0x2A
GPL2_720		equ		0x50
DSPEC1_720		equ		0xDF		; D=step rate 6ms, F=head unload time (480ms)
DSPEC2_720		equ		0x11		; 8=32ms head load time (very safe), 1=no DMA
DSPD_720		equ		0x02		; speed 250kB/s (CR1=1, CR0=0, DRV=x)
SpT_720			equ		0x09		; sectors per track (9)


; Data buffer memory organization: starts at address 0x100 (addressed using indirect addressing scheme)

; ***********************************************************************************************************
; *						 Definitions of the variables in the ACCESS memory (0x000..0x05f)					*
; ***********************************************************************************************************

CMDLOC		UDATA_ACS	0x000		; location for command queue for FDC
; *** Command buffer memory organization
;	output: ACCESS bank, sequence of the bytes (or words) to send to controller (offset 0x00-0x0F)
;	intput: ACCESS bank, sequence of the bytes (or words) read from the controller (offset 0x10-0x1F)
CMDWLOC			res		0x010		; 0x00		; location of the queue which is written to the FDC
CMDRLOC			res		0x010		; 0x10		; location of the bytes which are read from FDC


SYSVAR		UDATA_ACS	0x020		; location for system variables
; *** FDC-related variables
MAINSTAT		res		0x001		; 0x20		; last read master status register of FDD
FDCQPOS			res		0x001		; 0x21		; FDC queue end pointer
QTEMP			res		0x001		; 0x22		; temporary argument storage for FDC queueing
FDCRUNPOS		res		0x001		; 0x23		; position of the pointer when writing sequence to the FDD controller
FDCERR			res		0x001		; 0x24		; floppy access error indicator, 0xff if error
MOTATO			res		0x001		; 0x25		; motor drive A timeout (0=no timeout/stopped)
MOTBTO			res		0x001		; 0x26		; motor drive B timeout (0=no timeout/stopped)
DORVAL			res		0x001		; 0x27		; current value of the DOR
CURDRV			res		0x001		; 0x28		; Current _physical_ disk drive (0=A, 1=B; always 0 for single drive config)
CURHOSTDRV		res		0x001		; 0x29		; current _logical_ host's disk drive (0=A, 1=B)
SPDCOU			res		0x001		; 0x2a		; Counter for speed retry, sometimes used as temp location
DSK0SPD			res		0x001		; 0x2b		; DSK*SPD contain speed current disk/disk drive is using
DSK1SPD			res		0x001		; 0x2c		; 0=500kB/s, 1=250kB/s
CURGPL1			res		0x001		; 0x2d		; current values of the diskette (SpT)/FDC (speed settings)
CURGPL2			res		0x001		; 0x2e
CURDSP1			res		0x001		; 0x2f
CURDSP2			res		0x001		; 0x30
CURDSPD			res		0x001		; 0x31
CURSPT			res		0x001		; 0x32
DSK0CHGFLG		res		0x001		; 0x33		; disk change flags for drives 0 and 1
DSK1CHGFLG		res		0x001		; 0x34
CYLPRM			res		0x001		; 0x35		; cylinder parameter in FDC format
SECPRM			res		0x001		; 0x36		; sector parameter in FDC format
FL0ERR			res		0x001		; 0x37		; floppy drive error flag. If subsystem is in compatible mode, and user inserts
FL1ERR			res		0x001		; 0x38		; disk with more than 3 FAT sectors, respective flag is set and all further access
												; to the respective floppy drive will immediately return error until diskette
												; is changed

; DSKIO dedicated variables
DSKIO_SEC_0		res		0x001		; 0x39		; logical sector to access (4 bytes)
DSKIO_SEC_1		res		0x001		; 0x3a
DSKIO_SEC_2		res		0x001		; 0x3b
DSKIO_SEC_3		res		0x001		; 0x3c
DSKIO_SpT		res		0x001		; 0x3d		; sectors per track (physical dimensions, 1 byte)
DSKIO_Heads		res		0x001		; 0x3e		; heads (physical dimensions, 1 byte)
DSKIO_NumOfSec	res		0x001		; 0x3f		; number of sectors to process
DIO_NumSecProcessed	res		0x001	; 0x30		; number of sectors actually processed
DIO_Retry_Count	res		0x001		; 0x41		; retry count for the operation
CURCYL0			res		0x001		; 0x42		; current cylinder (head) positions of _physical_ drives
CURCYL1			res		0x001		; 0x43
DSKIO_CRC		res		0x001		; 0x44		; data transfer CEC counting variable

; host command and media access variables
DRVPRM			res		0x001		; 0x45		; _physical_ drive number (also with head ID in FDC routines)
HOSTCMD			res		0x001		; 0x46		; current host command

; temporary variables
TMP				res		0x001		; 0x47		; temporary location, used in many subs
TEMPREQ			res		0x001		; 0x48		; temporary location to store status byte of FDC

; FDD format routine variables/temporary variables
FMTPRM			res		0x001		; 0x49		; variables used in FORMAT routine (sometimes used as temp)
FMTPR1			res		0x001		; 0x4a
FMTPR2			res		0x001		; 0x4b
FMTPR3			res		0x001		; 0x4c
FMTTMP1			res		0x001		; 0x4d
FMTTMP2			res		0x001		; 0x4e
FMTTMP3			res		0x001		; 0x4f

; Timeout variables used in status check
TIMOUT1			res		0x001		; 0x50
TIMOUT2			res		0x001		; 0x51
TIMOUT3			res		0x001		; 0x52

; Counter (16-bit)
RECLENL			res		0x001		; 0x53
RECLENH			res		0x001		; 0x54

; Memory cells to hold delay counter
DELAY1			res		0x001		; 0x55		; used within delay counting code
DELAY2			res		0x001		; 0x56		; used within delay counting code
DELAY3			res		0x001		; 0x57		; used within delay counting code
									; 0x58..0x5f free
; --- Last variable space offset within ACCESS bank should be = 0x5F


; ***********************************************************************************************************
; *						 Definitions of the variables in the GPR memory: STORM (0xe00)						*
; ***********************************************************************************************************
STORMVAR		UDATA	0xe00		; Variables for Storage Manager (bank 0x0e), all are temporary
ST_TOTSEC0		res		0x001		; 0xe00		; total number of sectors on the hard drive
ST_TOTSEC1		res		0x001		; 0xe01
ST_TOTSEC2		res		0x001		; 0xe02
ST_TOTSEC3		res		0x001		; 0xe03
ST_MAXCLU0		res		0x001		; 0xe04		; total number of clusters in the LVOL
ST_MAXCLU1		res		0x001		; 0xe05
ST_SPCMAX		res		0x001		; 0xe06		; (used for input conversion and ckecking)
									; 			; contains maximal allowed value for sectors per cluster
ST_FSMAX		res		0x001		; 0xe07		; (used for input conversion and ckecking)
									; 			; contains maximal allowed value for FAT sector count
ST_PRTTYP		res		0x001		; 0xe08		; current partition type (0=FAT12, non-0=FAT16)
ST_PRTCNF		res		0x001		; 0xe09		; number of partitions configured
ST_DIRENT0		res		0x001		; 0xe0a		; number of directory enties (fixed)
ST_DIRENT1		res		0x001		; 0xe0b
ST_LVSTRT0		res		0x001		; 0xe0c		; (next) LVOL start LBA
ST_LVSTRT1		res		0x001		; 0xe0d
ST_LVSTRT2		res		0x001		; 0xe0e
ST_LVSTRT3		res		0x001		; 0xe0f
ST_DIRSEC		res		0x001		; 0xe10		; number of directory sectors (fixed)
ST_TMPSTR		res		0x008		; 0xe11		; temporary location to get input string to (8 bytes max)


; ***********************************************************************************************************
; *							 Definitions of the variables in the GPR memory (0xf00)							*
; ***********************************************************************************************************
MATHVAR			UDATA	0xf00		; variables for math routines
; Divident
DVDT_0			res		0x001		; 0xf00		; Dividend
DVDT_1			res		0x001		; 0xf01
DVDT_2			res		0x001		; 0xf02
DVDT_3			res		0x001		; 0xf03
DVSR_0			res		0x001		; 0xf04		; Divisor
DVSR_1			res		0x001		; 0xf05
DVSR_2			res		0x001		; 0xf06
DVSR_3			res		0x001		; 0xf07
RMDR_0			res		0x001		; 0xf08		; Remainder
RMDR_1			res		0x001		; 0xf09
RMDR_2			res		0x001		; 0xf0a
RMDR_3			res		0x001		; 0xf0b
DIVCTR			res		0x001		; 0xf0c		; Counter for division operation

; *** Variables for Storage Manager (bank 0x0f), temporary
ST_CHARMIN		res		0x001		; 0xf0d		; minimum char to accept for GetCharAndValidate (e.g. 0x31)
ST_CHARMAX		res		0x001		; 0xf0e		; maximal char to accept for GetCharAndValidate (e.g. 0x36)

; *** Storage subsystem configuration variables (bank 0x0f)
FTMP			res		0x001		; 0xf0f		; temporary variable for bank 0x0f
OPMODE			res		0x001		; 0xf10		; Storage subsystem operation mode (loaded from EEPROM location 0 on subsystem init)
												; bit 0=1: floppy drive is in compatible mode
												; bit 1=1: floppy phantom drive is disabled
												; bit 2=1: in device order floppies go first
												; bit 6=0: floppy disk subsystem is disabled
												; bit 7=0: hard disk subsystem is disabled

; *** the following sections should be contiguous as they are addresses indirectly as an array
; Floppies variables
NUMLFLP			res		0x001		; 0xf11		; number of logical floppies configured
NUMHFLP			res		0x001		; 0xf12		; numper of physical floppes found
; The followng section should be contiguous as it is being addressed indirectly
FD0SPT			res		0x001		; 0xf13		; current number of sectors per track for drive 0
FD0HD			res		0x001		; 0xf14		; current number of heads for drive 0
FD1SPT			res		0x001		; 0xf15		; current number of sectors per track for drive 1
FD1HD			res		0x001		; 0xf16		; current number of heads for drive 1

; HDDs variables
NUMHDD			res		0x001		; 0xf17		; number of hard disk drives found
; The followng section should be contiguous as it is being addressed indirectly
HD0CYL			res		0x002		; 0xf18		; number of cylinders for HDD0 (word)
HD0SPT			res		0x001		; 0xf1a		; sector per track for HDD0
HD0HD			res		0x001		; 0xf1b		; heads for HDD0
HD1CYL			res		0x002		; 0xf1c		; number of cylinders for HDD1 (word)
HD1SPT			res		0x001		; 0xf1e		; sector per track for HDD1
HD1HD			res		0x001		; 0xf1f		; heads for HDD1

; LVOL variables
NUMLVL			res		0x001		; 0xf20		; number of logical volumes (LVOLs) configured
; The followng section should be contiguous as it is being addressed indirectly
LVOL0D			res		0x001		; 0xf21		; HDD number (0/1) where LVOL1 is residing
LVOL0T			res		0x001		; 0xf22		; file system type (from MBR), 0x01=FAT12
LVOL0BG			res		0x004		; 0xf23		; absolute starting sectors of the volume on the respective HDD (4 bytes, dword)
LVOL0SZ			res		0x004		; 0xf27		; size of the volume (in sectors) on the respective HDD (4 bytes, dword)
LVOL1D			res		0x001		; 0xf2b		; HDD number (0/1) where LVOL1 is residing
LVOL1T			res		0x001		; 0xf2c		; file system type (from MBR), 0x01=FAT12
LVOL1BG			res		0x004		; 0xf2d		; absolute starting sectors of the volume on the respective HDD (4 bytes, dword)
LVOL1SZ			res		0x004		; 0xf31		; size of the volume (in sectors) on the respective HDD (4 bytes, dword)

HDDSEL			res		0x001		; 0xf35		; used temporarily by the STORM utility to store currently selected hard disk drive device
FTMP1			res		0x001		; 0xf36		; temp usage
CFGCHGFLG		res		0x001		; 0xf37		; configuration change flag (for STORM)
; --- Last variable space offset within 0x0f bank = 0xf37
