;********************************************************************************
;*																				*
;*								License and T&C									*
;*																				*
;*			GR8BIT storage subsystem microcontroller microcode version 3.0		*
;*			     This software is copyleft 2011-12 by Eugeny Brychkov			*
;*																				*
;* You can use and modify its source and executable, given two conditions are	*
;* met: (a) you keep this Copyleft statement in the source code, and you do		*
;* not distribute software and resulting executables on the commercial basis	*
;* and for the purposes, not related to GR8BIT platform, without written		*
;* permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)					*
;*																				*
;********************************************************************************
;*			File name: GR8STORM_FLOPPY_LOW_LEVEL.INC							*
;********************************************************************************

; This file represents routines for low level floppy controller/drive access
GR8STORM_FLOPPY_LOW_LEVEL	CODE


; ***********************************************************************************************************
; *											Set speed for the FDC											*
; ***********************************************************************************************************
SetSpeeds:
	; set channel speed
	movf	CURDSPD,W,ACCESS
	movwf	PORTA,ACCESS
	clrf	PORTE,ACCESS		; activate /LDCR

	movlw	0xF8				; /LDOR=1, A=0, /CS=1: FDD controller operations register selected
	movwf	PORTC,ACCESS
	bcf		LATC,4,ACCESS		; /WRI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /WRI impulse width: 125ns (>60ns as per datasheet)
	bsf		LATC,4,ACCESS		; /WRI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS		; release /WRI
	setf	PORTA,ACCESS		; release data bus
	setf	PORTE,ACCESS		; release /LDCR

	; SPECIFY command (global attribute about disk drives' modes and times)
	call	NewFDCRequest
	movlw	0x03				; SPECIFY command
	call	AddToFDCQueue
	movf	CURDSP1,W,ACCESS
	call	AddToFDCQueue
	movf	CURDSP2,W,ACCESS
	bcf		WREG,0,ACCESS		; set DMA bit on
	call	AddToFDCQueue
	call	RunCommand
	return


; ***********************************************************************************************************
; *								Set speed-related variables to 1.44MB media									*
; ***********************************************************************************************************
Set144SpeedVals:
	movlw	GPL1_144
	movwf	CURGPL1,ACCESS
	movlw	GPL2_144
	movwf	CURGPL2,ACCESS
	movlw	DSPEC1_144
	movwf	CURDSP1,ACCESS
	movlw	DSPEC2_144
	movwf	CURDSP2,ACCESS
	movlw	DSPD_144
	movwf	CURDSPD,ACCESS
	movlw	SpT_144
	movwf	CURSPT,ACCESS
	return

; ***********************************************************************************************************
; *								Set speed-related variables to 720KB media									*
; ***********************************************************************************************************
Set720SpeedVals:
	movlw	GPL1_720
	movwf	CURGPL1,ACCESS
	movlw	GPL2_720
	movwf	CURGPL2,ACCESS
	movlw	DSPEC1_720
	movwf	CURDSP1,ACCESS
	movlw	DSPEC2_720
	movwf	CURDSP2,ACCESS
	movlw	DSPD_720
	movwf	CURDSPD,ACCESS
	movlw	SpT_720
	movwf	CURSPT,ACCESS
	return


; ***********************************************************************************************************
; *											Soft reset the FDC												*
; ***********************************************************************************************************
SoftReset:
	; Soft resetting FDD controller: LDOR=0/1, WR=1/0/1, 8 bit data bus
	clrf	PORTA,ACCESS		; SRST=0 (reset), DMAEN=0 (no DMA/IRQ)
	movlw	0x78				; /LDOR=0, A=0, /CS=1: FDD controller operations register selected
	movwf	PORTC,ACCESS
	bcf		LATC,4,ACCESS		; /WRI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /WRI impulse width: 125ns (>60ns as per datasheet)
	bsf		LATC,4,ACCESS		; /WRI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
	setf	PORTA,ACCESS		; setting data bus to high state

	movlw	0xFF
	call	Wait_ms				; wait 255 ms

	call	ReadMainStatRegFDC

	movlw	0x04				; release reset, DMAEN=0 (disabled)
	movwf	PORTA,ACCESS
	movwf	DORVAL,ACCESS
	movlw	0x78				; /LDOR=0, A=0, /CS=1: FDD controller operations register selected
	movwf	PORTC,ACCESS
	bcf		LATC,4,ACCESS		; /WRI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /WRI impulse width: 125ns (>60ns as per datasheet)
	bsf		LATC,4,ACCESS		; /WRI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
	setf	PORTA,ACCESS
	; reset complete

	movlw	0xFF
	call	Wait_ms				; wait 255 ms

	call	ReadMainStatRegFDC
	call	SenseInterruptCmd

	setf	TIMOUT1,ACCESS		; timeout: 255 loops
CheckAgain:
	call	ReadMainStatRegFDC	; read main status register
	btfsc	MAINSTAT,4,ACCESS
	bra		NotReady			; go there if bit 4 = 1 (command is executing)
	btfss	MAINSTAT,7,ACCESS
	bra		NotReady			; go there if bit 7 = 0 (FDC is not ready for data exchange)
	btfss	MAINSTAT,6,ACCESS
	bra		ReadyToWrite		; go there if bit 6 = 0 (ok to write to FDC)
	; FDC has something to say here (do not put port A to input as it does not matter what FDC is saying)
	setf	PORTA,ACCESS		; port A - high level
	movlw	0xD9				; /FDDCS(0)+A0(1) = data register selected
	movwf	PORTC,ACCESS
	bcf		LATC,3,ACCESS		; /RDI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /RDI impulse width: 125ns (>90ns as per datasheet)
	bsf		LATC,3,ACCESS		; /RDI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
NotReady:
	decfsz	TIMOUT1,1,ACCESS
	bra		CheckAgain
	bra		SoftReset		; reset again after timeout

ReadyToWrite:
	; set up data rate
	call	SetSpeeds
	return

; ***********************************************************************************************************
; *								FDC operations: DMA enable and disable routines								*
; ***********************************************************************************************************
EnableDMA:
	; *** setting PORTB mode and output value
	movlw	0x04
	movwf	LATB,ACCESS			; /DACK and TC lines are inactive
	movlw	0x03
	movwf	TRISB,ACCESS		; DI8 and DI9 are inputs, other pins are outputs

	bsf		DORVAL,3,ACCESS		; Enable DMA
	bra		DORWrite
DisableDMA:
	bcf		DORVAL,3,ACCESS		; Disable DMA
	call	DORWrite

	setf	LATB,ACCESS			; outputs 0xff
	clrf	TRISB,ACCESS		; port B is output
	return

DORWrite:
	movf	DORVAL,W,ACCESS
	movwf	PORTA,ACCESS
	movlw	0x78				; /LDOR=0, A=0, /CS=1: FDD controller operations register (AT/EISA mode) selected
	movwf	PORTC,ACCESS
	bcf		LATC,4,ACCESS		; /WRI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /WRI impulse width: 125ns (>60ns as per datasheet)
	bsf		LATC,4,ACCESS		; /WRI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
	setf	PORTA,ACCESS
	return


; ***********************************************************************************************************
; *										Initialize the floppy drive											*
; ***********************************************************************************************************
; in:	CURDRV = current drive
; out:	FDCERR = set if error occured at any of the init steps
InitDrive:
	; Turn motor on
	call	MotorOnSelect
	call	SetSpeeds

	; recalibrating drive
	call	Recalibrate
	btfsc	FDCERR,0,ACCESS
	bra		RetError

	; Seek to track 79
	call	NewFDCRequest
	movlw	0x0F				; SEEK command
	call	AddToFDCQueue
	movf	CURDRV,W,ACCESS
	call	AddToFDCQueue
	movlw	0x4F				; new cylinder number: 79
	call	AddToFDCQueue
	call	RunCommandNoResult	; no result phase
	btfsc	FDCERR,0,ACCESS
	bra		RetError

; --- Seek to track 0, drive 0
	call	NewFDCRequest
	movlw	0x0F				; SEEK command
	call	AddToFDCQueue
	movf	CURDRV,W,ACCESS
	call	AddToFDCQueue
	movlw	0x00				; new cylinder number: 00
	call	AddToFDCQueue
	call	RunCommandNoResult	; no result phase
	btfsc	FDCERR,0,ACCESS
	bra		RetError
	bcf		STATUS,C,ACCESS
	return

RetError:
	call	MotorOff	; start motor off countdown for current drive
	bsf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *								Run FDC command with data (with execute phase)								*
; ***********************************************************************************************************
; in:	none
; out:	status bytes in bank 0 starting offset 0x40,
;		FDCRUNPOS=number of results read, FDCERR<>0 if error
RunCommandWithData:
	call	EnableDMA
	call	MotorOnSelect
	clrf	FDCERR,ACCESS
	call	CommandPhase
	call	ExecutePhase
	call	ResultPhase
	call	MotorOff
	call	DisableDMA
	return

; ***********************************************************************************************************
; *								Run FDC command without data (no execute phase)								*
; ***********************************************************************************************************
; in:	none
; out:	status bytes in bank 0 starting offset 0x40,
;		FDCRUNPOS=number of results read, FDCERR<>0 if error
RunCommand:
	clrf	FDCERR,ACCESS
	call	MotorOnSelect
	call	CommandPhase
	call	ResultPhase
	call	MotorOff
	return



; ***********************************************************************************************************
; *						Run FDC command without data and result (seek, recalibrate)							*
; ***********************************************************************************************************
RunCommandNoResult:
	; send command (which has no result phase - seek and recalibrate)
	clrf	FDCERR,ACCESS
	call	EnableDMA
	call	MotorOnSelect
	call	CommandPhase

	; now waiting for interrupt signal (very long timeout)
	clrf	TIMOUT3,ACCESS			; long timout!
	clrf	TIMOUT2,ACCESS			; long timout!
	clrf	TIMOUT1,ACCESS
IRQLoop:
	btfsc	PORTB,0,ACCESS			; bit 0 = IRQ line
	bra		InterruptOk
	decfsz	TIMOUT1,1,ACCESS
	bra		IRQLoop
	decfsz	TIMOUT2,1,ACCESS
	bra		IRQLoop
	decfsz	TIMOUT3,1,ACCESS
	bra		IRQLoop

	; timeout
	setf	FDCERR,ACCESS

InterruptOk:
	call	DisableDMA

	; checking source of interrupt, and condition
	call	SenseInterruptCmd

	movlw	0xe0
	andwf	CMDRLOC,W,ACCESS
	sublw	0x20
	btfss	STATUS,Z,ACCESS
	setf	FDCERR,ACCESS			; should be 001xxxxx for successful completion of command

	call	MotorOff
	call	DisableDMA
	return


; ***********************************************************************************************************
; *								FDC command phases description: Execute phase								*
; ***********************************************************************************************************
ExecutePhase:
	; *** pre-clearing temporary storage (for TC bit)
	clrf	TMP,ACCESS

	; *** clear byte counter
	clrf	RECLENL,ACCESS
	movlw	0x02
	movwf	RECLENH,ACCESS			; byte count in the record (0x200)

	; *** clear timeout counter
	clrf	TIMOUT3,ACCESS
	clrf	TIMOUT2,ACCESS
	clrf	TIMOUT1,ACCESS		; long timeout = 0x10000 loops

	; wait for execution phase start
ExecLoop:
	; *** reading DMA channel (DI9)
	btfsc	PORTB,1,ACCESS		; checking DMA line
	bra		DMADataAvailable

	; *** check for interrupt IRQ=1 (exec phase complete)
	btfsc	PORTB,0,ACCESS
	bra		ExecReturn			; execution phase finished: return

	; and wait in loop
	decfsz	TIMOUT1,1,ACCESS
	bra		ExecLoop
	decfsz	TIMOUT2,1,ACCESS
	bra		ExecLoop
	decfsz	TIMOUT3,1,ACCESS
	bra		ExecLoop

ExecError:
	; timeout occured
	call	SoftReset			; perform soft reset of the controller
ExecErrReturn:
	setf	FDCERR,ACCESS
ExecReturn:
	return

DMADataAvailable:
	; *** respond with /DACK and TC if it is set
	movff	TMP,LATB			; TMP has bit 2 reset (/DACK active)
								; and TC set appropriately
	btfsc	CMDWLOC,0,ACCESS		; which direction DMA works?
	bra		DMAWriteDir				; command's bit 0=1 => data will be supplied to FDC
	; DMA read from FDC
	call	ReadDataRegFDC		; read data register
	movwf	POSTINC1,ACCESS			; write data to buffer and increase FSR0
	bra		EndDMACycle
DMAWriteDir:
	; DMA write to FDC
	movf	POSTINC1,W,ACCESS
	call	WriteDataRegFDC

EndDMACycle:
	movlw	0x04
	movwf	PORTB,ACCESS		; deactivate /DMA and TC lines

	; *** clearing temporary storage (for TC bit)
	clrf	TMP,ACCESS

	; *** checking for last byte (to set TC active)
	movf	RECLENL,W,ACCESS
	sublw	0x02
	bnz		ExecLoop	;NotLastByte
	movf	RECLENH,W,ACCESS
	sublw	0x01
	bnz		ExecLoop	;NotLastByte

	; *** we are here when RECLENH:RECLENL=1:1
	; meaning last byte to transfer
	bsf		TMP,3,ACCESS		; raise bit 3 to set TC in next (last) loop
	bra		ExecLoop


; ***********************************************************************************************************
; *								FDC command phases description: Result phase								*
; ***********************************************************************************************************
; in:	none
; out	CMDRLOC area is filled with status bytes (max 16)
ResultPhase:
	lfsr	FSR0,CMDRLOC			; pointer to receiving queue
	clrf	FDCRUNPOS,ACCESS

	clrf	TIMOUT2,ACCESS			; long timout! ; 0x08
	clrf	TIMOUT1,ACCESS			; timeout = 0x800 loops
ReadLoop:
	call	ReadMainStatRegFDC
	btfsc	MAINSTAT,7,ACCESS		; 0x80, RQM
	bra		DataRegReady

	decfsz	TIMOUT1,1,ACCESS
	bra		ReadLoop
	decfsz	TIMOUT2,1,ACCESS
	bra		ReadLoop

	; timeout - exit
ReadErrorExit:
	setf	FDCERR,ACCESS
	return

DataRegReady:
	btfss	MAINSTAT,4,ACCESS		; 0x10, CB
	return							; result phase complete - return without error

	btfss	MAINSTAT,6,ACCESS		; DIO - should be 1, i.e. data ready for reading
	bra		ReadLoop

	call	ReadDataRegFDC
	movwf	POSTINC0,ACCESS			; write byte to the indirectly addressed location

	incf	FDCRUNPOS,1,ACCESS		; increment current pointer to the queue, increase pointer
	movf	FDCRUNPOS,W,ACCESS
	sublw	0x10
	bnz		ReadLoop				; keep filling buffer if no buffer overflow (16 bytes)
	bra		ReadErrorExit			; result buffer overrun (max 16 result bytes)


; ***********************************************************************************************************
; *								FDC command phases description: Command phase								*
; ***********************************************************************************************************
; Writes queue contents to the FDC from CMDWLOC location
CommandPhase:
	lfsr	FSR0,CMDWLOC			; pointer to sending queue
	clrf	FDCRUNPOS,ACCESS

	clrf	TIMOUT2,ACCESS
	clrf	TIMOUT1,ACCESS			; timeout = 0x800 loops

WriteLoop:
	call	ReadMainStatRegFDC
	btfsc	MAINSTAT,7,ACCESS		; 0x80, RQM
	bra		WriteRQMReady
CPhWait:
	decfsz	TIMOUT1,1,ACCESS
	bra		WriteLoop
	decfsz	TIMOUT2,1,ACCESS
	bra		WriteLoop

	; timeout - exit
WriteErrorExit:
	setf	FDCERR,ACCESS
	return

WriteRQMReady:
	btfsc	MAINSTAT,6,ACCESS
	bra		CPhWait					;WriteErrorExit		; DIO=1 means controller has unexpectedly something to say

	movf	POSTINC0,W,ACCESS		; read byte from the indirectly addressed location and increase pointer
	call	WriteDataRegFDC

	incf	FDCRUNPOS,1,ACCESS		; increment current pointer to the queue
	movf	FDCRUNPOS,W,ACCESS
	subwf	FDCQPOS,W,ACCESS
	bnz		WriteLoop
	return						; FDCERR is preserved


; ***********************************************************************************************************
; *											Read FDC data regiser											*
; ***********************************************************************************************************
; out: data byte in W
ReadDataRegFDC:
	setf	PORTA,ACCESS		; port A - high level
	movlw	0xD9				; /FDDCS+A0(1) = data register selected
	movwf	PORTC,ACCESS
	bcf		LATC,3,ACCESS		; /RDI activated (important: do not use PORTC as bcf is rmw operation)
	setf	TRISA,ACCESS
	nop							; /RDI impulse duration before data read: 187.5ns (>90ns Trd, as per datasheet)
	movf	PORTA,W,ACCESS
	bsf		LATC,3,ACCESS		; /RDI deactivated (important: do not use PORTC as bcf is rmw operation)
	clrf	TRISA,ACCESS
	setf	PORTC,ACCESS
;	movwf	DATAVAL,ACCESS
	return


; ***********************************************************************************************************
; *											Write FDC data regiser											*
; ***********************************************************************************************************
; in:	W = data byte to write
WriteDataRegFDC:
	movwf	PORTA,ACCESS
	movlw	0xD9				; /FDDCS+A0(1) = data register selected
	movwf	PORTC,ACCESS
	bcf		LATC,4,ACCESS		; /WRI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /WRI impulse width: 125ns (>60ns as per datasheet)
	bsf		LATC,4,ACCESS		; /WRI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
	return


; ***********************************************************************************************************
; *									Read FDC master status regiser (MSR)									*
; ***********************************************************************************************************
; out: status byte in W
ReadMainStatRegFDC:
	setf	PORTA,ACCESS		; port A - high level
	movlw	0xD8				; /FDDCS+A0(0) = master register selected
	movwf	PORTC,ACCESS
	bcf		LATC,3,ACCESS		; /RDI activated (important: do not use PORTC as bcf is rmw operation)
	setf	TRISA,ACCESS
	nop							; /RDI impulse duration before data read: 187.5ns (>90ns Trd, as per datasheet)
	movf	PORTA,W,ACCESS
	clrf	TRISA,ACCESS
	bsf		LATC,3,ACCESS		; /RDI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
	movwf	MAINSTAT,ACCESS
	return


; ***********************************************************************************************************
; *									Add byte to the FDC command queue										*
; ***********************************************************************************************************
; in: WREG=byte to add
AddToFDCQueue:
	movwf	QTEMP,ACCESS
	clrf	FSR0H,ACCESS		; indirect addressing bank 0
	movf	FDCQPOS,W,ACCESS
	addlw	CMDWLOC
	movwf	FSR0L,ACCESS		; visible in all banks
	movf	QTEMP,W,ACCESS
	movwf	INDF0,ACCESS		; write to the indirectly addressed location
	incf	FDCQPOS,1,ACCESS
	return


; ***********************************************************************************************************
; *											Clear FDC command queue											*
; ***********************************************************************************************************
NewFDCRequest:
	clrf	FDCQPOS,ACCESS
	return

; ***********************************************************************************************************
; *										Sense interrupt subroutine											*
; ***********************************************************************************************************
SenseInterruptCmd:
	call	NewFDCRequest
	movlw	0x08				; SENSE INTERRUPT command
	call	AddToFDCQueue
	call	RunCommand
	return


; ***********************************************************************************************************
; *								Select floppy drive and turn its motor on									*
; ***********************************************************************************************************
; in:	CURDRV = current _physical_ drive #
MotorOnSelect:
	clrf	TMP,ACCESS
	movf	DORVAL,W,ACCESS
	btfss	CURDRV,0,ACCESS
	bra		Drv0

	btfsc	DORVAL,5,ACCESS		; bit of drive motor running
	incf	TMP,1,ACCESS		; TMP to change to 1	

	clrf	MOTBTO,ACCESS		; no drive B timeout - keep it running
	andlw	0xFC				; mask drive select bits
	iorlw	0x21				; turn drive B motor on and select drive B
	bra		WriteDOR

Drv0:
	btfsc	DORVAL,4,ACCESS		; bit of drive motor running
	incf	TMP,1,ACCESS		; TMP to change to 1	

	clrf	MOTATO,ACCESS		; no drive A timeout - keep it running
	andlw	0xFC				; mask drive select bits
	iorlw	0x10				; turn drive A motor on and select drive A

WriteDOR:
	movwf	DORVAL,ACCESS		; save current DOR value
	movwf	PORTA,ACCESS
	movlw	0x78				; /LDOR=0, A=0, /CS=1: select FDD controller operations register
	movwf	PORTC,ACCESS
	bcf		LATC,4,ACCESS		; /WRI activated (important: do not use PORTC as bcf is rmw operation)
	nop							; /WRI impulse width: 62.5ns (>60ns as per datasheet)
	bsf		LATC,4,ACCESS		; /WRI deactivated (important: do not use PORTC as bcf is rmw operation)
	setf	PORTC,ACCESS
	setf	PORTA,ACCESS

	btfsc	TMP,0,ACCESS
	return						; skip drive spin up delay if TMP<>0 (selected drive motor was already running)

	movlw	0xFF
	call	Wait_ms
	movlw	0xFF
	call	Wait_ms
	movlw	0xFF
	call	Wait_ms
	movlw	0xFF
	call	Wait_ms				; wait 255*4 ~ 1 second
	return


; ***********************************************************************************************************
; *								Set up counter to turn floppy drive motor off								*
; ***********************************************************************************************************
; (motor is being turned off in interrup routine)
; in:	CURDRV = current drive #
; corrupt: none
MotorOff:
	btfss	CURDRV,0,ACCESS
	bra		MotorCountDownDrv0

	setf	MOTBTO,ACCESS		; drive B timeout set
	return

MotorCountDownDrv0:
	setf	MOTATO,ACCESS		; drive A timeout set
	return

;********************************************************************************
;*			END OF: GR8STORM_FLOPPY_LOW_LEVEL.INC								*
;********************************************************************************