;********************************************************************************
;*																				*
;*								License and T&C									*
;*																				*
;*			GR8BIT storage subsystem microcontroller microcode version 3.0		*
;*			     This software is copyleft 2011-12 by Eugeny Brychkov			*
;*																				*
;* You can use and modify its source and executable, given two conditions are	*
;* met: (a) you keep this Copyleft statement in the source code, and you do		*
;* not distribute software and resulting executables on the commercial basis	*
;* and for the purposes, not related to GR8BIT platform, without written		*
;* permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)					*
;*																				*
;********************************************************************************
;*			File name: GR8STORM_MANAGER.INC										*
;********************************************************************************

; This file represents routines for GR8BIT Storage Manager
GR8STORM_MANAGER	CODE


; ***********************************************************************************************************
; *									GR8BIT storage management utility: main									*
; ***********************************************************************************************************
GR8STORM:
	; respond to host that command has started, and check confirmation code
	movlw	0xAB
	call	WaitForHostRead				; return 0xAB to the host - command has started
	btfss	STATUS,C,ACCESS				; A0 address line state during host request
	goto	NoCommandToRun				; access to port base+1 is not assumed this time, aborting MC command

	call	WaitForHostWrite			; read confirmation code
	btfss	STATUS,C,ACCESS				; address A0 is set to 1 (reset MC operation process)
	goto	NoCommandToRun				; in this case return 0xAA

	sublw	0xEC						; pass word to start initialization of the FDD hardware
	btfss	STATUS,Z,ACCESS
	goto	NoCommandToRun

	movlb	0x0f						; change to bank 0x0f otherwise it will not work
	clrf	CFGCHGFLG,BANKED			; no vital config changes were made so far

	; display main utility message
	lfsr	FSR0,STORM_InitStr-STORM_Pointers
	call	DisplayFSR0


; ***********************************************************************************************************
; *											Level 1: top menu												*
; ***********************************************************************************************************
TopMenuDisplay:
	lfsr	FSR0,STORM_TopMenu-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x34
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		FinishSTORM

	lfsr	FSR2,CallTableL1-STORM_Pointers
	call	CallSubInW
	bnc		TopMenuDisplay

FinishSTORM:
	; display "reboot needed" string in case vital changes were made
	lfsr	FSR0,STORM_RebootSystem-STORM_Pointers
	btfsc	CFGCHGFLG,0,BANKED
	call	DisplayFSR0

	; exiting STORM
	call	SendLastChar				; finish output to host

	; send reboot code (0xCC = no reboot)
	movlb	0x0f
	movlw	0xCC
	btfsc	CFGCHGFLG,0,BANKED
	movlw	0x00
	call	WaitForHostRead
	goto	HostWaitMainLoop


; ***********************************************************************************************************
; *									Level 2: Media configuration menu										*
; ***********************************************************************************************************
MediaConfigMenuSelect:
	lfsr	FSR0,STORM_MediaConfigMenu-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x35
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		FinishMediaConfigMenu

	lfsr	FSR2,CallTableL2MediaConfig-STORM_Pointers
	call	CallSubInW
	bnc		MediaConfigMenuSelect

	bcf		STATUS,C,ACCESS				; get to higher level menu
FinishMediaConfigMenu:
	return


; ***********************************************************************************************************
; *											Level 2: Message menu											*
; ***********************************************************************************************************
MessageMenuSelect:
	lfsr	FSR0,STORM_MessageMenu-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x32
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		FinishMessageMenu

	lfsr	FSR2,CallTableL2MessageMenu-STORM_Pointers
	call	CallSubInW
	bnc		MessageMenuSelect

	bcf		STATUS,C,ACCESS				; get to higher level menu
FinishMessageMenu:
	return


; ***********************************************************************************************************
; *										Level 2: Operation mode menu										*
; ***********************************************************************************************************
OpModeMenuSelect:
	lfsr	FSR0,STORM_OpModeMenu-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x36
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		FinishOpModeMenu

	lfsr	FSR2,CallTableL2OpMode-STORM_Pointers
	call	CallSubInW
	bc		OpMMExit					; user selected to exit from the menu

	movlb	0x0f
	setf	CFGCHGFLG,BANKED			; user selected other option: reboot will be required
	bra		OpModeMenuSelect

OpMMExit:
	bcf		STATUS,C,ACCESS				; get to higher level menu
FinishOpModeMenu:
	return


; ***********************************************************************************************************
; *										Enumerate hardware in the GR8STORM									*
; ***********************************************************************************************************
Enumerate:
	; display horizontal line at the top
	lfsr	FSR0,STORM_Enumerate-STORM_Pointers
	call	DisplayFSR0
	; enumerate
	call	EnumerateHardware
	; display another horizontal line at the bottom
	lfsr	FSR0,STORM_Enumerate-STORM_Pointers
	call	DisplayFSR0
	; wait until FDD motors stop
	call	WaitFDDMotorStop
	; display "press any key" and wait for keystroke
	call	PressAnyKeyToContinue
	bcf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *								Level 3 END: Operation mode subroutines										*
; ***********************************************************************************************************
FloppyEnaDisa:
	btg		OPMODE,6,BANKED
OpSubsReturnNoC:
	btfss	STATUS,C,ACCESS
	return
HDDEnaDisa:
	btg		OPMODE,7,BANKED
	bra		OpSubsReturnNoC
DeviceOrderSet:
	btg		OPMODE,2,BANKED
	bra		OpSubsReturnNoC
FDDModeSet:
	btg		OPMODE,0,BANKED
	bra		OpSubsReturnNoC
PhantomModeSet:
	btg		OPMODE,1,BANKED
	bra		OpSubsReturnNoC
WriteOpModeEEPROM:
	; we are here to write EEPROM data
	clrf	EEADR,ACCESS
	clrf	EEADRH,ACCESS				; EEPROM address = 0
	movf	OPMODE,W,BANKED
	movwf	EEDATA,ACCESS
	call	WriteDataToEEPROM
	bra		OpSubsReturnNoC


; ***********************************************************************************************************
; *											Partition manager												*
; ***********************************************************************************************************
PartitionHDDSelect:

	call	SelectHDDandListPartitions
	bnc		HardDiskWasSelectedOK
	bra		PartMgrReturn1

PartMgrRetNC1FSR:
	call	DisplayFSR0
PartMgrReturn1:
	bcf		STATUS,C,ACCESS					; C flag is reset - do not get to higher level menu
	return

HardDiskWasSelectedOK:
	; Ask question "Repartition the disk?", and get uppercase Y response
	lfsr	FSR0,STORM_RepartitionDiskQ-STORM_Pointers
	call	PromptYN
	bc		PartMgrReturn1
	bz		GoRePartitionDisk

PartMgrAbortedExit:
	; display "Aborted" and return to partition manager's menu
	lfsr	FSR0,STORM_Aborted-STORM_Pointers
	bra		PartMgrRetNC1FSR

GoRePartitionDisk:
	; display string "Disk size is"
	lfsr	FSR0,STORM_DiskSizeIs-STORM_Pointers
	call	DisplayFSR0

	; *** get HDD physical configuration (Cylinders/SpT/Hd)
	lfsr	FSR0,HD0CYL				; load pointer to the start of HDD HW properties table
	movf	HDDSEL,W,BANKED
	andlw	0x01
	movwf	HDDSEL,BANKED			; mask bit 0
	addwf	WREG,W,ACCESS
	addwf	WREG,W,ACCESS			; current hard disk drive number * 4
	addlw	0x03					; skip number of cylinders and sectors per track (scan back)
	addwf	FSR0L,1,ACCESS			; and add it to the pointer to the FSR0 no need to correct
									; FSR0H as it does not span 256 byte boundary
	movff	INDF0,DSKIO_Heads
	movff	POSTDEC0,DVDT_0
	movff	INDF0,DSKIO_SpT
	movff	POSTDEC0,DVSR_0
	movlb	0x0f
	clrf	DVDT_1,BANKED
	clrf	DVSR_1,BANKED
	call	MUL16					; SpT*Heads

	movff	RMDR_0,DVDT_0
	movff	RMDR_1,DVDT_1
	movff	POSTDEC0,DVSR_1
	movff	POSTDEC0,DVSR_0
	call	MUL16					; CylNum*SpT*Heads

	; preserve total number of sectors into ST_TOTSECx
	movff	RMDR_0,ST_TOTSEC0
	movff	RMDR_1,ST_TOTSEC1
	movff	RMDR_2,ST_TOTSEC2
	movff	RMDR_3,ST_TOTSEC3

	; convert size in sectors in RMDR into KBytes and display
	call	ConvertSectorsToKBandDisplay

	; print CR/LF
	lfsr	FSR0,STORM_CR-STORM_Pointers
	call	DisplayFSR0

	; clear partition space
	call	ClearBufForFormat			; clears one sector's space for MBR
	; prepare boot sectors
	call	PrepareBootSectors			; four sectors, at 0x300, 0x500, 0x700 and 0x900

	; Main MBR signature
	movlw	LOW STORM_MBR1
	movwf	TBLPTRL,ACCESS
	movlw	HIGH STORM_MBR1
	movwf	TBLPTRH,ACCESS
	lfsr	FSR1,0x100
	call	MoveROMDataToFSR1
	; Signature before partition table
	movlw	LOW STORM_MBR2
	movwf	TBLPTRL,ACCESS
	movlw	HIGH STORM_MBR2
	movwf	TBLPTRH,ACCESS
	lfsr	FSR1,0x2b8
	call	MoveROMDataToFSR1
	; "55 AA" MBR signature
	movlw	LOW STORM_MBR3
	movwf	TBLPTRL,ACCESS
	movlw	HIGH STORM_MBR3
	movwf	TBLPTRH,ACCESS
	lfsr	FSR1,0x2fe
	call	MoveROMDataToFSR1

	; set partition's record location in MBR
	lfsr	FSR1,0x2be						; start of the record for partition entry 0
	movlw	0x00
	; initialize number of partitions configures
	movlb	0x0e
	movwf	ST_PRTCNF,BANKED				; number of partitions configured = 0
	; initialize offset of the start of next (first) partition
	clrf	ST_LVSTRT3,BANKED
	clrf	ST_LVSTRT2,BANKED
	clrf	ST_LVSTRT1,BANKED
	movf	DSKIO_SpT,W,ACCESS
	movwf	ST_LVSTRT0,BANKED				; (starts at the C/H/S=0/1/1)
	movlb	0x0f

BuildMBRLoop:
	; display "Partition number..."
	lfsr	FSR0,STORM_PartitionNumber-STORM_Pointers
	call	DisplayFSR0

	call	SetParametersFAT12				; set default partition type to FAT12
	movf	FSR1L,W,ACCESS
	sublw	0xbe
	bz		GoForNextPartitionEntry			; first parition must be FAT12

	; we are here if not first partition
	sublw	0x00-0x40
	bz		AcceptAndCreate					; last partition was processed - go for MBR update

;GoNextPartitionEntry:
	; now ask which type of partition is wanted
	lfsr	FSR0,STORM_WhichPartitionWanted-STORM_Pointers
	movlw	0x31
	movwf	ST_CHARMIN,BANKED
	addlw	0x32
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		CancelRePartition1			; return with C=1

	movff	WREG,ST_PRTTYP				; set partition type: 0=FAT12, 1=FAT16
	tstfsz	WREG,ACCESS
	call	SetParametersFAT16			; in case type is set to 1 fill with FAT16 parameters

GoForNextPartitionEntry:
	; set variables to the maximum
	movff	ST_FSMAX,DIO_NumSecProcessed	; sectors per FAT
	movff	ST_SPCMAX,WREG
	call	CalculateSecClus				; sectors per cluster
	call	CalcMaxCluster

PartitionInputMenuDisplay:
	; display "Fixed parameters..." with defined variables
	lfsr	FSR0,STORM_MenuPartitionParameters-STORM_Pointers
	movlw	0x31
	movwf	ST_CHARMIN,BANKED
	addlw	0x34
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bnc		NoAbortExit
	goto	PartMgrAbortedExit			; exit this menu with "Aborted" string

NoAbortExit:
	movwf	FTMP,BANKED					; preserve selection
	sublw	0x01
	bnc		AcceptPartition				; options 2 and 3
	movf	FTMP,W,BANKED				; restore selection

	; we are here with options 0 and 1 only
	lfsr	FSR2,CallTableTunePartition-STORM_Pointers
	call	CallSubInW
	bra		PartitionInputMenuDisplay

AcceptPartition:
	movwf	DIO_Retry_Count,ACCESS		; temporarily preserve (1-selection) in DIO_Retry_Count
										; 0xff=accept and next entry, 0xfe=accept and create

	; accept selected configuration and configure next partition if there's any
	call	ConfigurePartition			; shifts FSR1 to next partition's entry
	bc		PartitionInputMenuDisplay	; work on the same partition if HDD size exceeded
	movlb	0x0e
	incf	ST_PRTCNF,1,BANKED			; increase number of partitions configured
	movlb	0x0f

	; check if option 4 was selected (Accept and Create)
	incf	DIO_Retry_Count,W,ACCESS
	bnz		AcceptAndCreate
	; check if all 4 entries were populated
	movff	ST_PRTCNF,WREG
	sublw	0x04
	bnz		BuildMBRLoop

AcceptAndCreate:
	; Ask question "Are you sure", and get uppercase Y response
	lfsr	FSR0,STORM_AreYouSure-STORM_Pointers
	call	PromptYN
	bc		PMAbExit1
	bnz		PMAbExit1

	; setting the flag that vital changes to storage subsystem were performed
	setf	CFGCHGFLG,BANKED			; no vital config changes were made so far

	; display string "Creating structures..."
	lfsr	FSR0,STORM_CreatingLVOLs-STORM_Pointers
	call	DisplayFSR0

	; writing MBR from buffer address 0x100
	clrf	DSKIO_SEC_0,ACCESS
	clrf	DSKIO_SEC_1,ACCESS
	clrf	DSKIO_SEC_2,ACCESS
	clrf	DSKIO_SEC_3,ACCESS			; sector 0 - MBR
	call	WriteOneNextControlSector
	bz		PartMgr_WriteMBROk

PartMgr_WriteSectorError:
	; display "Error writing sector"
	lfsr	FSR0,STORM_SectorWriteError-STORM_Pointers
	bra		PartMgrRetNC1FSR
CancelRePartition1:
	return
PMAbExit1:
	goto	PartMgrAbortedExit

PartMgr_WriteMBROk:
	; display "." to reset host dialogue timeout counter
	movlw	"M"
	call	SendCharToHost
	; move MBR to the memory address 0xb00
	lfsr	FSR0,0x100
	lfsr	FSR1,0xb00
MBRMoveLoop:
	movff	POSTINC0,POSTINC1
	movlw	0x03
	subwf	FSR0H,W,ACCESS
	bnz		MBRMoveLoop

	; set up MBR pointer into FSR1 pointing to the OS type
	lfsr	FSR1,0xcbe+4
	; set FSR2 to the first boot sector
	lfsr	FSR2,0x300

WriteNextPartitionEntry:
	; get OS type, and set ST_PRTTYP
	decf	INDF1,W,ACCESS			; if FAT12 then WREG will be 0x00 (0x01-1=0x00)
	btfss	STATUS,Z,ACCESS
	movlw	0x01
	movff	WREG,ST_PRTTYP
	; shifting MBR pointer to the start sector field
	movlw	0x04
	addwf	FSR1L,1,ACCESS
	; set new write starting sector to the start of the volume
	movff	POSTINC1,DSKIO_SEC_0
	movff	POSTINC1,DSKIO_SEC_1
	movff	POSTINC1,DSKIO_SEC_2
	movff	POSTINC1,DSKIO_SEC_3		; sector number = start of the LVOL

	; write structures to the LVOL
	call	WriteLVOLStructures
	bc		PartMgr_WriteSectorError	; error writing structures

	; move MBR pointer (FSR1) to the next entry's OS type field
	movlw	0x08
	addwf	FSR1L,1,ACCESS

	; use SPDCOU as partition (entry) temporary counter
	movlb	0x0e
	decf	ST_PRTCNF,1,BANKED
	movlb	0x0f
	bnz		WriteNextPartitionEntry
	; return no error
	bcf		STATUS,C,ACCESS
CancelRePartition:
	return


; ***********************************************************************************************************
; *											Write LVOL structures											*
; ***********************************************************************************************************
; in: ST_PRTTYP is set to partition type, DSKIO_SEC_x is set to start absolute sector
; in: FSR2 is set to _valid_ image of the respective boot sector (0x300,0x500,0x700,0x900)
; in: HDDSEL is set to hard disk drive to write to
; out: flag C=1 if write error
WriteLVOLStructures:
	; and now proceed with cycle to write structures for each configured LVOL
	; move boot sector to the 0x100
	lfsr	FSR0,0x100
BootMoveLoop:
	movff	POSTINC2,POSTINC0
	movlw	0x03
	subwf	FSR0H,W,ACCESS
	bnz		BootMoveLoop

	; make next sector after new boot (0x300 location) a first FAT sector
	movlw	0xf0
	movwf	POSTINC0,ACCESS				; media descriptor
	movlw	0xff
	movwf	POSTINC0,ACCESS				; cluster 0/1
	movwf	POSTINC0,ACCESS				; cluster 0/1
	; following DIRSEC definition is important!
	movlw	0x10
	movff	WREG,ST_DIRSEC				; ST_DIRSEC to contain number of directory sectors (FAT12)

	movff	ST_PRTTYP,WREG				; get OS type of current partition
	movf	WREG,W,ACCESS				; and test it to be 0 (FAT12)
	bz		SecClearLoop				; no third 0ffh for FAT12
	movlw	0xff
	movwf	POSTINC0,ACCESS				; cluster 0/1 (for FAT16 implementation)
	movlw	0x20
	movff	WREG,ST_DIRSEC				; now ST_DIRSEC contains 32 directory sectors (FAT16)

	; clear remainder of the FAT sector
SecClearLoop:
	movlw	0x00
	movwf	POSTINC0,ACCESS
	movlw	0x05
	subwf	FSR0H,W,ACCESS
	bnz		SecClearLoop

	; copying sectors per FAT to ST_TOTSEC3:ST_TOTSEC2 and to ST_TOTSEC1:ST_TOTSEC0
	movff	0x116,ST_TOTSEC0
	movff	0x117,ST_TOTSEC1
	movff	0x116,ST_TOTSEC2
	movff	0x117,ST_TOTSEC3

	; display "." to reset host dialogue timeout counter
	movlw	"."
	call	SendCharToHost

	; set bootsector and first FAT sector location
	movff	HDDSEL,DRVPRM				; get HDD physical number
	movlw	0x02
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 2 (boot+first FAT sector)
	movwf	HOSTCMD,ACCESS				; write operation = 2
	; and write them (2) to hard drive
	call	DSKIO_HDD					; write physical harddrive
	sublw	0xCC
	bz		WriteLVOLStructuresMBROk

WriteLVOLStructuresError:
	; return with error
	bsf		STATUS,C,ACCESS
	return

WriteLVOLStructuresMBROk:
	; display "." to reset host dialogue timeout counter
	movlw	"B"
	call	SendCharToHost

	; emptying sector area at 0x100 (we do not need boot sector anymore)
	call	ClearBufForFormat

FirstFATCopyLoop:
	; check counter at the beginning (-1 cycle!)
	movlb	0x0e
	decf	ST_TOTSEC0,1,BANKED
	movlw	0x00
	subwfb	ST_TOTSEC1,1,BANKED
	movf	ST_TOTSEC0,W,BANKED
	iorwf	ST_TOTSEC1,W,BANKED
	movlb	0x0f
	bz		EndOfFirstFatWrite

	; writing remainder of the first FAT copy
	call	WriteOneNextControlSector
	bnz		WriteLVOLStructuresError
	bra		FirstFATCopyLoop

EndOfFirstFatWrite:
	; display "." to reset host dialogue timeout counter
	movlw	"F"
	call	SendCharToHost

	; move 4 bytes from master FAT sector to the buffer
	movff	0x300,0x100
	movff	0x301,0x101
	movff	0x302,0x102
	movff	0x303,0x103

	; write first sector of second copy of FAT
	call	WriteOneNextControlSector
	bnz		WriteLVOLStructuresError

	; emptying sector area at 0x100 (we do not need boot sector anymore)
	call	ClearBufForFormat

SecondFATCopyLoop:
	; check counter at the beginning (-1 cycle!)
	movlb	0x0e
	decf	ST_TOTSEC2,1,BANKED
	movlw	0x00
	subwfb	ST_TOTSEC3,1,BANKED
	movf	ST_TOTSEC2,W,BANKED
	iorwf	ST_TOTSEC3,W,BANKED
	movlb	0x0f
	bz		EndOfSecondFatWrite

	; writing remainder of the first FAT copy
	call	WriteOneNextControlSector
	bnz		WriteLVOLStructuresError
	bra		SecondFATCopyLoop

EndOfSecondFatWrite:
	; display "." to reset host dialogue timeout counter
	movlw	"f"
	call	SendCharToHost
DirWrite:
	; write directory empty sectors
	call	WriteOneNextControlSector
	btfss	STATUS,Z,ACCESS
	goto	WriteLVOLStructuresError

	movlb	0x0e
	decf	ST_DIRSEC,1,BANKED
	movlb	0x0f
	bnz		DirWrite

	; display "." to reset host dialogue timeout counter
	movlw	"D"
	call	SendCharToHost

	; return - no error
	bcf		STATUS,C,ACCESS
	return



; ***********************************************************************************************************
; *									Write one sector from 0x100	to HDD										*
; ***********************************************************************************************************
; in: DSKIO_SEC_x, HDDSEL
; out: Z=1 if success
WriteOneNextControlSector:
	movff	HDDSEL,DRVPRM				; get HDD physical number
	; DSKIO_SEC_x: (writing to next sector)
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 1
	movlw	0x02
	movwf	HOSTCMD,ACCESS				; write operation = 2
	; and write them (2) to hard drive
	call	DSKIO_HDD					; write physical harddrive, does not affect FSR1 and FSR2!
	sublw	0xCC
	return


; ***********************************************************************************************************
; *			Configure partition in the MBR abd spartition recovery info and, compose boot sector for it		*
; ***********************************************************************************************************
; in: FSR1 points to the MBR record destination
; in: the following values are set - ST_PRTTYP (partition type), ST_PRTCNF (number of partition)
; out: if C=0: FSR1 points to next MBR record destination
; out: if C=1: FSR1 not changed
ConfigurePartition:
	; fill MBR entry
	movlw	0x80
	movwf	INDF1,ACCESS			; by default partition is active
	movlw	0x04
	addwf	FSR1L,1,ACCESS			; shift to OS type byte

	; set FSR2 to the partition recovery info in MBR
	lfsr	FSR2,0x180				; recovery info starts at offset 0x080
	movlb	0x0e
	movf	ST_PRTCNF,W,BANKED
	addwf	WREG,W,ACCESS
	addwf	WREG,W,ACCESS
	addwf	WREG,W,ACCESS			; partition number * 8
	addwf	FSR2L,1,ACCESS			; now points to the respective place of
									; recovery info in MBR
	; fill variables in recovery area of MBR
	movff	DSKIO_NumOfSec,POSTINC2			; sectors per cluster
	movff	DIO_NumSecProcessed,POSTINC2	; sectors in FAT copy

	; set MBR entry OS type
	movf	ST_PRTTYP,W,BANKED		; test for 0 (FAT12)
	movlb	0x0f
	movlw	0x01
	bz		PartitionIsFAT12
	movlw	0x06
PartitionIsFAT12:
	movwf	INDF1,ACCESS			; set OS type
	; move MBR pointer to the location of start LBA data
	movlw	0x04
	addwf	FSR1L,1,ACCESS
	; copy start of the volume to the start field of MBR
	movff	ST_LVSTRT0,POSTINC1
	movff	ST_LVSTRT1,POSTINC1
	movff	ST_LVSTRT2,POSTINC1
	movff	ST_LVSTRT3,POSTINC1

	; go constructing boot sector image
	call	ConstructBootSector

	; now completing MBR record with size of the volume
	movff	RMDR_0,POSTINC1
	movff	RMDR_1,POSTINC1
	movff	RMDR_2,POSTINC1
	movff	RMDR_3,POSTINC1		
	; and calculate position for next LVOL (in ST_LVSTRTx)
	movff	ST_LVSTRT0,WREG
	addwf	RMDR_0,1,BANKED
	movff	ST_LVSTRT1,WREG
	addwfc	RMDR_1,1,BANKED
	movff	ST_LVSTRT2,WREG
	addwfc	RMDR_2,1,BANKED
	movff	ST_LVSTRT3,WREG
	addwfc	RMDR_3,1,BANKED						; add sector count to base (new offset)

	; now we check if volume exceeds size of the disk (ST_TOTSECx)
	movff	ST_TOTSEC0,DVDT_0
	movff	ST_TOTSEC1,DVDT_1
	movff	ST_TOTSEC2,DVDT_2
	movff	ST_TOTSEC3,DVDT_3					; copy disk sector number of the current volume to DVDT
	movf	RMDR_0,W,BANKED
	subwf	DVDT_0,1,BANKED
	movf	RMDR_1,W,BANKED
	subwfb	DVDT_1,1,BANKED
	movf	RMDR_2,W,BANKED
	subwfb	DVDT_2,1,BANKED
	movf	RMDR_3,W,BANKED
	subwfb	DVDT_3,1,BANKED						; substract new offset from the total number of sectors
	bnc		ExceedsDiskSize

	movff	RMDR_0,DVDT_0
	movff	RMDR_1,DVDT_1
	movff	RMDR_2,DVDT_2
	movff	RMDR_3,DVDT_3
	movff	DSKIO_SpT,DVSR_0
	clrf	DVSR_1,BANKED
	clrf	DVSR_2,BANKED
	clrf	DVSR_3,BANKED
	call	DIV32								; divide this offset by the sectors per track

	movlw	0x02
	addwf	DVDT_0,1,BANKED
	movlw	0x00
	addwfc	DVDT_1,1,BANKED
	addwfc	DVDT_2,1,BANKED
	addwfc	DVDT_3,1,BANKED						; add 2 to the result
	call	MUL32								; multiply by the sectors per track
	movff	RMDR_0,ST_LVSTRT0
	movff	RMDR_1,ST_LVSTRT1
	movff	RMDR_2,ST_LVSTRT2
	movff	RMDR_3,ST_LVSTRT3					; and set as offset for next volume
	bcf		STATUS,C,ACCESS
	return

ExceedsDiskSize:
	lfsr	FSR0,STORM_PartitionExceeds-STORM_Pointers
	call	DisplayFSR0
	movlw	0x100-0x10
	addwf	FSR1L,1,ACCESS						; revert pointer back to current record's start
	bsf		STATUS,C,ACCESS
	return

; ***********************************************************************************************************
; *								Construct boot sector given preset values									*
; ***********************************************************************************************************
; in: volume structure set, DIO_NumSecProcessed (SpC), DIO_NumSecProcessed (FAT sectors)
; in: the following values are set:
;		ST_PRTCNF = partition entry number selected (set by user dialogue)
;		DSKIO_NumOfSec = sectors per cluster (set by user dialogue/restoring recover value)
;		ST_DIRENT1:ST_DIRENT0 = number of directory entries (SetParameters)
;		DIO_NumSecProcessed = sectors per FAT (set by user dialogue/restoring recover value)
;		ST_DIRSEC = number of sectors for directory (SetParameters)
;		ST_MAXCLU1:ST_MAXCLU0 = maximal number of clusters (CalcMaxCluster)
;		DSKIO_SpT = sectors per track (DSKIO_HDD)
;		DSKIO_Heads = heads (DSKIO_HDD)
;		ST_PRTTYP = partition type (SetParameters)
; out: RMDR = sector count for volume
ConstructBootSector:
	; calculate FSR0 pointing to respective boot sector's first modifiable entry
	lfsr	FSR0,0x30d
	movff	ST_PRTCNF,WREG
	addwf	WREG,W,ACCESS
	addwf	FSR0H,1,ACCESS

	; at this stage we have half of MBR filled,
	; and FSR0 pointing to the beginning of respective boot sector's modifiable entry
	movff	DSKIO_NumOfSec,POSTINC0	; sectors per cluster
	movlw	0x03
	addwf	FSR0L,1,ACCESS			; now points to the DirEnt
	movff	ST_DIRENT0,POSTINC0
	movff	ST_DIRENT1,POSTINC0		; copy DirEnt data to the boot sector

	; now we will calculate number of sectors for disk
	; formula is (MaxClus-([(1{boot}+2*SecFat+16{dir})/SpC)]+1)*SpC, [] denotes fix part
	movf	DIO_NumSecProcessed,W,ACCESS
	movwf	DVDT_0,BANKED
	movlw	0x00
	bnz		SecFatNot0x100
	movlw	0x01
SecFatNot0x100:
	movwf	DVDT_1,BANKED
	rlcf	DVDT_0,1,BANKED
	rlcf	DVDT_1,1,BANKED
	bcf		DVDT_0,0,BANKED						; SecFAT*2
	; add directory sector count
	movff	ST_DIRSEC,WREG
	addwf	DVDT_0,1,BANKED
	movlw	0x00
	addwfc	DVDT_1,1,BANKED
	; add boot sector
	incf	DVDT_0,1,BANKED
	addwfc	DVDT_1,1,BANKED
	movff	DSKIO_NumOfSec,DVSR_0
	clrf	DVSR_1,BANKED						; divide by Sectors per cluster
	call	DIV16								; DVDT_0 contains fix part of division

	incf	DVDT_0,W,BANKED						; increase cluster count used by control structures
	movlb	0x0e
	subwf	ST_MAXCLU0,W,BANKED
	movff	WREG,DVDT_0
	movlw	0x00
	subwfb	ST_MAXCLU1,W,BANKED
	movlb	0x0f
	movwf	DVDT_1,BANKED						; now the only action is to multiply by SpC
												; which is already present in DVSR
	call	MUL16

	; now total sectors in the LVOL is contained in RMDR
	; if partition is FAT16, set "size in sectors" in boot sector to 0x0000
	; and set actual size in BigSize location (+0x20)
	movff	ST_PRTTYP,WREG
	movf	WREG,W,ACCESS
	bz		NotABigSizeVolume					; FAT12 detected fill 16-bit volume size
	; bigsize detected, fill 32-bit volume size instead
	clrf	POSTINC0,ACCESS
	clrf	POSTINC0,ACCESS						; set TotalSect of the boot sector to 0x0000
	movlw	0x0b
	addwf	FSR0L,1,ACCESS						; shift to the bigzie location in the boot sector
	movff	RMDR_0,POSTINC0
	movff	RMDR_1,POSTINC0
	movff	RMDR_2,POSTINC0
	movff	RMDR_3,POSTINC0						; put bigsize in
	movlw	0x100-0x0e
	addwf	FSR0L,1,ACCESS						; return back to SecPerFAT position of boot sector
	bra		SizeIsThere

NotABigSizeVolume:
	; copy 16-bit size to the TotalSect field
	movff	RMDR_0,POSTINC0
	movff	RMDR_1,POSTINC0
	; and shift to SecPerFAT
	incf	FSR0L,1,ACCESS

SizeIsThere:
	; put sectors per FAT into boot sector
	movf	DIO_NumSecProcessed,W,ACCESS
	movwf	POSTINC0,ACCESS
	movlw	0x00
	btfsc	STATUS,Z,ACCESS
	movlw	0x01
	movwf	POSTINC0,ACCESS						; 0x100 if lower byte is 0
	; now put sectors per track
	movff	DSKIO_SpT,POSTINC0
	clrf	POSTINC0,ACCESS
	; now put heads
	movff	DSKIO_Heads,POSTINC0
	clrf	POSTINC0,ACCESS						; boot sector is complete
	; compose volume serial number
	movlw	0x0b
	addwf	FSR0L,1,ACCESS						; shift to serial number field
	movff	TMR1L,RECLENL
	movff	TMR1H,RECLENH			; read 16-bit timer TMR1 (prescaled 1:2)
	call	TransformNumber
	movff	RECLENL,POSTINC0
	movff	RECLENH,POSTINC0		; lower word of the serial number
	; save lower SN word to recovery info
	movff	RECLENL,POSTINC2
	movff	RECLENH,POSTINC2

	movff	TMR0L,RECLENL
	movff	TMR0H,RECLENH			; read 16-bit timer TMR0 (prescaled 1:8)
	call	TransformNumber
	movff	RECLENL,POSTINC0
	movff	RECLENH,POSTINC0		; higher word of the serial number
	; save higher SN word to recovery info
	movff	RECLENL,POSTINC2
	movff	RECLENH,POSTINC2

	; set file system type in boot sector
	movlw	0x04
	addwf	FSR0L,1,ACCESS			; shift to the last char in "FAT1x" string
	movff	ST_PRTTYP,WREG
	movf	WREG,1,ACCESS			; test WREG
	movlw	"2"
	btfss	STATUS,Z,ACCESS
	movlw	"6"
	movwf	POSTINC0,ACCESS
	return


; ***********************************************************************************************************
; *								Set FAT12 partition configuration properties								*
; ***********************************************************************************************************
SetParametersFAT12:
	movlb	0x0e
	movlw	0x04
	movwf	ST_SPCMAX,BANKED			; max sectors per cluster for FAT12 is set to 0x04 (2^4=16)
	movlw	0x0c
	movwf	ST_FSMAX,BANKED				; max number of FAT sectors for FAT12 is set to 0xc (12)
	movlw	0xfe
	movwf	ST_DIRENT0,BANKED
	movlw	0x00
	movwf	ST_DIRENT1,BANKED			; 254 dir entries
	movwf	ST_PRTTYP,BANKED			; partition type = 0 (FAT12)
	movlw	0x10
	movwf	ST_DIRSEC,BANKED			; number of directory sectors = 16
	movlb	0x0f
	return


; ***********************************************************************************************************
; *								Set FAT16 partition configuration properties								*
; ***********************************************************************************************************
SetParametersFAT16:
	movlb	0x0e
	movlw	0x06
	movwf	ST_SPCMAX,BANKED			; max sectors per cluster for FAT12 is set to 0x06 (2^6=64)
	clrf	ST_FSMAX,BANKED				; max number of FAT sectors for FAT12 is set to 0x100 (0)
	movlw	0x00
	movwf	ST_DIRENT0,BANKED
	movlw	0x02
	movwf	ST_DIRENT1,BANKED			; 512 dir entries
	movlw	0x20
	movwf	ST_DIRSEC,BANKED			; number of directory sectors = 32
	movlw	0x01
	movwf	ST_PRTTYP,BANKED			; partition type = 1 (FAT16)
	movlb	0x0f
	return


; ***********************************************************************************************************
; *				Gets string of FTMP defined meaningful characters into temp buffer ST_TMPSTR				*
; ***********************************************************************************************************
; corrupts TMP
GetTempString:
	movlw	0x02
	call	SendCharToHost				; request host to read string
	movlw	0x03						; only 3 characters
	call	SendCharToHost				; inform host to get only 3 chars
	lfsr	FSR0,ST_TMPSTR

GetTempStrLoop:
	call	WaitSTORMInput
	bc		EndOfTempString				; finish output if host signalled CTRL-BREAK

	movf	WREG,W,ACCESS
	bz		EndOfTempString				; write received 0x00 at last

	; checking if limit of 2 chars is exceeded
	movf	FTMP,1,BANKED
	bz		GetTempStrLoop				; string character count exceeded - go get chars without
										; writing to buffer till end of string (0x00)
	movwf	TMP,ACCESS					; preserve char
	; checking for character to be numerical
	movlw	0x2f
	cpfsgt	TMP,ACCESS
	bra		GetTempStrLoop				; if < 0x30
	movlw	0x3a
	cpfslt	TMP,ACCESS
	bra		GetTempStrLoop				; if > 0x3a

	movf	TMP,W,ACCESS
	movwf	POSTINC0,ACCESS
	decf	FTMP,1,BANKED
	bra		GetTempStrLoop

EndOfTempString:
	clrf	POSTINC0,ACCESS
	clrf	POSTINC0,ACCESS
	clrf	POSTINC0,ACCESS				; last 3 bytes of the string = 0x00
	return


; ***********************************************************************************************************
; *			Get temp string and convert it to number using first 3 decimal digits in the string				*
; ***********************************************************************************************************
; return 0 if string was empty
; out: DVDT_1:DVDT_0
GetTempStringAndConvertToNumber:
	call	GetTempString
	; convert
	clrf	DVDT_0,BANKED
	clrf	DVDT_1,BANKED
	movlw	0x1							; multiplier
	movwf	TMP,ACCESS

	movff	ST_TMPSTR+2,WREG			; get second character of the string
	movf	WREG,W,ACCESS				; test for 0
	bz		GoChar2

	addlw	0x100-0x30
	addwf	DVDT_0,1,BANKED
	movlw	0x0a
	movwf	TMP,ACCESS					; multiplier is 10 now

GoChar2:
	movff	ST_TMPSTR+1,WREG			; get second character of the string
	movf	WREG,W,ACCESS				; test for 0
	bz		GoChar1

	addlw	0x100-0x30
	mulwf	TMP,ACCESS					; multiply by multiplier (1 or 10)
	movf	PRODL,W,ACCESS				; 8 bit result
	addwf	DVDT_0,1,BANKED				; final representation of the entered number
	movlw	0xa
	mulwf	TMP,ACCESS
	movff	PRODL,TMP					; multiply multiplier by 10
GoChar1:
	movff	ST_TMPSTR,WREG				; get first character of the string
	movf	WREG,W,ACCESS				; test for 0
	bz		Char1IsZero					; 0x00 instead of character (empty string entered)
	addlw	0x100-0x30
	mulwf	TMP,ACCESS					; multiply by multiplier (1 or 10)
	movf	PRODL,W,ACCESS
	addwf	DVDT_0,1,BANKED
	movf	PRODH,W,ACCESS
	addwfc	DVDT_1,1,BANKED				; final representation of the entered number
Char1IsZero:
	return


; ***********************************************************************************************************
; *								Get sector per FAT in the range (3..ST_FSMAX)								*
; ***********************************************************************************************************
SetSecFAT:
	; display "SecFAT (3..X) >"
	lfsr	FSR0,STORM_OptionSecFAT-STORM_Pointers
	call	DisplayFSR0

	; set up input data for GetTempString
	movlw	0x3
	movwf	FTMP,BANKED					; 3 characters only!
	; and get the string checked for digits
	call	GetTempStringAndConvertToNumber

	; get maximal value, and check if it's 0x00. If it is,
	; minimal allowed value to check against will be 0x01
	movff	ST_FSMAX,WREG
	movf	WREG,1,ACCESS				; test ST_FSMAX for 0
	bz		DoNotCheckAgainst0			; if 0x00 is allowed value, do not check against it

	movf	DVDT_0,W,BANKED
	bz		PrintInvalidInput
DoNotCheckAgainst0:
	dcfsnz	DVDT_0,W,BANKED				; =1?
	bra		PrintInvalidInput
	dcfsnz	WREG,W,ACCESS				; =2?
	bra		PrintInvalidInput

	; checking for maximal value
	movff	ST_FSMAX,WREG
	movf	WREG,1,ACCESS				; test ST_FSMAX for 0
	bz		DoNotCheckMaxSpFValue		; if 0x00 is allowed value, do not max at all

	bsf		STATUS,C
	subfwb	DVDT_0,W,BANKED
	bnc		PrintInvalidInput			; go there if DVDT_0 > ST_FSMAX

DoNotCheckMaxSpFValue:
	; now checks are over, update SpF value
	movff	DVDT_0,DIO_NumSecProcessed
	return
PrintInvalidInput:
	; display "Invalid input"
	lfsr	FSR0,STORM_InvalidInput-STORM_Pointers
	call	DisplayFSR0
	return


; ***********************************************************************************************************
; *									Display ST_FSMAX in decimal format										*
; ***********************************************************************************************************
; display ST_FSMAX in decimal format
DisplaySecFATMax:
	movff	ST_FSMAX,DVDT_0
	clrf	DVDT_1,BANKED
	movf	DVDT_0,1,BANKED
	btfsc	STATUS,Z,ACCESS
	incf	DVDT_1,1,BANKED
	call	PrintWordInDecimal
	return


; ***********************************************************************************************************
; *									Display ST_SPCMAX in decimal format										*
; ***********************************************************************************************************
DisplaySecClusMax:
	movff	ST_SPCMAX,DVDT_0
	clrf	DVDT_1,BANKED
	call	PrintWordInDecimal
	return


; ***********************************************************************************************************
; *						 Get sectors per cluster (power of 2, capped by ST_SPCMAX)							*
; ***********************************************************************************************************
SetSecClus:
	; display "SecClus (0..X) >"
	lfsr	FSR0,STORM_OptionSecClus-STORM_Pointers
	; set MIN and MAX
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movff	ST_SPCMAX,ST_CHARMAX
	addwf	ST_CHARMAX,1,BANKED			; add 0x30 to the number to get digit
	call	GetCharAndValidate			; get char
	bc		SetSecClusRet

	call	CalculateSecClus			; calculate number from power of 2
	bcf		STATUS,C,ACCESS
SetSecClusRet:
	return


; ***********************************************************************************************************
; *						 Calculate sectors per cluster basing on input of power of 2						*
; ***********************************************************************************************************
; in: WREG=power of 2
; out: DSKIO_NumOfSec=sectors per cluster
CalculateSecClus:
	; counter is in WREG
	clrf	DSKIO_NumOfSec,ACCESS
	incf	DSKIO_NumOfSec,1,ACCESS		; initially 1 sector per cluster
	movf	WREG,W,ACCESS
	bz		SecClusNoShift				; do not shift if counter = 0
SecClusShift:
	rlcf	DSKIO_NumOfSec,1,ACCESS
	bcf		DSKIO_NumOfSec,0,ACCESS
	decfsz	WREG,1,ACCESS
	bra		SecClusShift
SecClusNoShift:
	return


; ***********************************************************************************************************
; *			Prepares raw boot sector images for 4 LVOLs at addresses 0x300, 0x500, 0x700 and 0x900			*
; ***********************************************************************************************************
PrepareBootSectors:
	; table pointer points to the LVOL boot sector image
	movlw	HIGH BootSectLVOL
	movwf	TBLPTRH,ACCESS
	movlw	LOW BootSectLVOL
	movwf	TBLPTRL,ACCESS
	; defining length of the code of the boot sector (remainder is filled with 0s
	movlw	BootSectLVOLEnd-BootSectLVOL	; counter of meaningful bytes
	movwf	RECLENL,ACCESS
	lfsr	FSR0,0x300						; address of the first boot sector location
BuildBootSector:
	tstfsz	RECLENL,ACCESS
	bra		PrepareFromTable			; table is still not exhausted
	; table exhausted - fill remainder with 0s
	movlw	0x00
	call	FillBootSectors
	bra		NextByteOfTable
PrepareFromTable:
	tblrd*+
	movf	TABLAT,W,ACCESS
	call	FillBootSectors
	decf	RECLENL,1,ACCESS			; minus one byte from table
NextByteOfTable:
	movf	FSR0H,W,ACCESS
	sublw	0x05
	bnz		BuildBootSector
	return


; ***********************************************************************************************************
; *			Puts value in WREG into the respective location of each of 4 LVOLs' boot sectors				*
; ***********************************************************************************************************
FillBootSectors:
	movwf	INDF0,ACCESS				; boot sector 0
	incf	FSR0H,1,ACCESS
	incf	FSR0H,1,ACCESS
	movwf	INDF0,ACCESS				; boot sector 1
	incf	FSR0H,1,ACCESS
	incf	FSR0H,1,ACCESS
	movwf	INDF0,ACCESS				; boot sector 2
	incf	FSR0H,1,ACCESS
	incf	FSR0H,1,ACCESS
	movwf	POSTINC0,ACCESS				; boot sector 3, pointer++
	movlw	0x100-0x6
	addwf	FSR0H,1,ACCESS				; restore FSRH
	return


; ***********************************************************************************************************
; *								Move data between RAM locations: FSR0 -> FSR1								*
; ***********************************************************************************************************
MoveROMDataToFSR1:
	tblrd*+
	movf	TABLAT,W,ACCESS
	bnz		NotEndOfData
	return
NotEndOfData:
	movwf	POSTINC1,ACCESS
	bra		MoveROMDataToFSR1


; ***********************************************************************************************************
; *	L3 END: Updates floppy disk boot sector with DOS2-type boot record preserving diskette physical config	*
; ***********************************************************************************************************
WriteDOS2BootSelect:
	lfsr	FSR0,STORM_FloppyDriveInsert-STORM_Pointers
	call	DisplayFSR0
	call	WaitSTORMInput
	bc		RetWriteDOS2Boot			; finish output if host signalled CTRL-BREAK

	sublw	0x0d
	bz		WriteFloppyBoot

	; print "Aborted" string if not ENTER was pressed
	lfsr	FSR0,STORM_Aborted-STORM_Pointers

RetWriteDOS2BootNoCDFSR:
	call	DisplayFSR0
RetWriteDOS2BootNoC:
	bcf		STATUS,C,ACCESS
RetWriteDOS2Boot:
	return

WriteFloppyBoot:
	movlb	0x0f
	; check if floppy drive 0 is present/configured
	movf	NUMHFLP,1,BANKED
	bnz		AtLeastOneFDDPresent

	; print "Floppy 0 is not present" string
	lfsr	FSR0,STORM_Floppy0Missing-STORM_Pointers
	bra		RetWriteDOS2BootNoCDFSR

AtLeastOneFDDPresent:
	; read data from floppy disk boot sector
	clrf	DRVPRM,ACCESS				; floppy drive 0
	clrf	DSKIO_SEC_0,ACCESS
	clrf	DSKIO_SEC_1,ACCESS			; boot sector (#0)
	movlw	0x01
	movwf	HOSTCMD,ACCESS				; sector read command
	movwf	DSKIO_NumOfSec,ACCESS
	call	DSKIO_FLOPPY				; read the sector
	call	WaitFDDMotorStop
	sublw	0xcc
	bz		SectorReadSuccess

SectorReadErrorDisplay:
	; print "Sector read error" string
	lfsr	FSR0,STORM_SectorReadError-STORM_Pointers
	bra		RetWriteDOS2BootNoCDFSR

SectorReadSuccess:
	; display "+"
	movlw	"+"
	call	SendCharToHost

	; set ROM pointer to the DOS2 boot sector
	movlw	HIGH BootSectDOS2
	movwf	TBLPTRH,ACCESS
	movlw	LOW BootSectDOS2
	movwf	TBLPTRL,ACCESS

	; copy first 13 bytes to the boot sector, including GR8BIT signature
	movlw	0x0b						; 11 bytes from the beginning of the boot sector, including signature
	lfsr	FSR0,0x100					; pointer to loaded boot sector
	call	CopyDataToBoot
	; skipping floppy disk configuration parameters
	movlw	0x13
	addwf	FSR0L,1,ACCESS					; (FSR0L will not span 256-byte limit)
	addwf	TBLPTRL,1,ACCESS				; shift pointers withint boot sector image in RAM and ROM to code area
	movlw	0x00
	addwfc	TBLPTRH,1,ACCESS
	; copy executable DOS2 code to the boot sector
	movlw	(EndOfBootSectDOS2-BootSectDOS2-0x1e)		; should be < than 256!
	call	CopyDataToBoot
	; and clear remaining area of the boot sector
KeepClearingBSArea:
	movf	FSR0H,W,ACCESS
	sublw	0x03
	bz		BootSectorAreaFilledOk		; 0x300 reached - end of sector area

	clrf	POSTINC0,ACCESS
	bra		KeepClearingBSArea

BootSectorAreaFilledOk:
	; display "+"
	movlw	"+"
	call	SendCharToHost
	; write data to the floppy disk boot sector
	movlw	0x02
	movwf	HOSTCMD,ACCESS				; write operation - 0x02
	call	DSKIO_FLOPPY				; read the sector
	; wait for FDD motor to stop
	call	WaitFDDMotorStop
	sublw	0xcc
	bz		RetWriteDOS2BootNoC

	; print "Sector write error" string
	lfsr	FSR0,STORM_SectorWriteError-STORM_Pointers
	bra		RetWriteDOS2BootNoCDFSR


; ***********************************************************************************************************
; *								Copy a portion of data from flash to MC RAM									*
; ***********************************************************************************************************
; in: WREG = record size, FSR0 = RAM buffer destination, TBLPRT = flash buffer source
CopyDataToBoot:
	tblrd*+
	movff	TABLAT,POSTINC0
	decfsz	WREG,1,ACCESS
	bra		CopyDataToBoot
	return


; ***********************************************************************************************************
; *					Writes data set into EEDATA to the EEPROM location set to EEADRH:EEADR					*
; ***********************************************************************************************************
WriteDataToEEPROM:
	bcf		EECON1,EEPGD,ACCESS			; point to data EEPROM memory, enable writes
	bcf		EECON1,CFGS,ACCESS			; access EEPROM
	bsf		EECON1,WREN,ACCESS			; enable writes

	bcf		INTCON,GIE,ACCESS			; ensuring interrupts are disabled
	movlw	0x55
	movwf	EECON2,ACCESS
	movlw	0xaa
	movwf	EECON2,ACCESS
	bsf		EECON1,WR,ACCESS			; initiate write to EEPROM

	; wait until write to EEPROM is complete
WaitEEPROMWrComplete:
	btfss	PIR2,EEIF,ACCESS
	bra		WaitEEPROMWrComplete

	bcf		PIR2,EEIF,ACCESS			; clear this bit
	bcf		EECON1,WREN,ACCESS			; disable writes to the eeprom
	return


; ***********************************************************************************************************
; *										L3 END: Clears message in EEPROM									*
; ***********************************************************************************************************
ClearMessageSet:
	movlw	0x01
	movwf	EEADR,ACCESS
	clrf	EEADRH,ACCESS				; EEPROM address = 1
	setf	EEDATA,ACCESS
	call	WriteDataToEEPROM			; set byte 1 of the EEPROM to 0xff (empty message)
	bcf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *								L3 END: Gets new message and sets it into the EEPROM						*
; ***********************************************************************************************************
NewMessageSet:
	lfsr	FSR0,STORM_NewMsgPrompt-STORM_Pointers
	call	DisplayFSR0
	movlw	0x01
	movwf	EEADR,ACCESS
	clrf	EEADRH,ACCESS				; EEPROM address = 1

	lfsr	FSR0,0x100					; will be reading string to the 0x100 buffer
	clrf	TMP,ACCESS					; clear counter

	movlw	0x18
	movwf	FTMP,BANKED					; 24 characters for the message
GetMessageLoop:
	call	WaitSTORMInput
	bc		MessageInterrupt			; finish output if host signalled CTRL-BREAK

	movf	WREG,W,ACCESS
	bz		EndOfNewMessage

	; checking if limit of 24 chars is exceeded
	movf	FTMP,1,BANKED
	bz		GetMessageLoop				; message character count exceeded - go get chars without
										; writing to EEPROM till end of string (0x00)
;	movwf	TMP,ACCESS
	sublw	0xfe						; end of output identifier, should not appear in the string!
	bnz		NotEndOfStringInMessage
	movlw	0xfe-"?"							; replace it with "?" character
NotEndOfStringInMessage:
	sublw	0xfe
	movwf	POSTINC0,ACCESS
	incf	TMP,1,ACCESS
	decf	FTMP,1,BANKED
	bra		GetMessageLoop

EndOfNewMessage:
	; now write message to the EEPROM
	lfsr	FSR0,0x100
WriteMsgEEPROM:
	movff	POSTINC0,EEDATA
	call	WriteDataToEEPROM
	incf	EEADR,1,ACCESS				; next EEPROM location
	decfsz	TMP,1,ACCESS
	bra		WriteMsgEEPROM

	bcf		STATUS,C,ACCESS
MessageInterrupt:
	return


; ***********************************************************************************************************
; *										L2: Performs diagnotic outputs										*
; ***********************************************************************************************************
DiagnosticOutputSelect:
	lfsr	FSR0,STORM_DiagnosticMenu-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x35
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		FinishDiagOutMenu

	lfsr	FSR2,CallTableL3DiagOutputs-STORM_Pointers
	call	CallSubInW
	bnc		DiagnosticOutputSelect

	bcf		STATUS,C,ACCESS				; get to higher level menu
FinishDiagOutMenu:
	return



; ***********************************************************************************************************
; *								L3 END: displays MBR and boot sectors of hard drive 0						*
; ***********************************************************************************************************
DisplayHDD0Struct:
	clrf	DRVPRM,ACCESS
	call	DisplayHDDStruct
	return


; ***********************************************************************************************************
; *								L3 END: displays MBR and boot sectors of hard drive 1						*
; ***********************************************************************************************************
DisplayHDD1Struct:
	movlw	0x01
	movwf	DRVPRM,ACCESS
	call	DisplayHDDStruct
	return


; ***********************************************************************************************************
; *								Displays HDD structures of the drive defined by DRVPRM						*
; ***********************************************************************************************************
DisplayHDDStruct:
	movlb	0x0f
	movf	HD0SPT,W,BANKED			; HDD0 sectors per track
	btfsc	DRVPRM,0,ACCESS
	movf	HD1SPT,W,BANKED			; HDD1 sectors per track
	bnz		DisplayHDDInstalled

	; display that there's no respective hard drive present
	lfsr	FSR0,STORM_NoHDDPresent-STORM_Pointers
	call	DisplayFSR0
DisplayHDD0Ret:
	bcf		STATUS,C,ACCESS
	return

DisplayHDDInstalled:
	movff	DRVPRM,HDDSEL			; preserve hard drive number is DRVPRM is corrupt in DSKIO_HDD
	; read and display MBR
	clrf	DSKIO_SEC_0,ACCESS
	clrf	DSKIO_SEC_1,ACCESS
	clrf	DSKIO_SEC_2,ACCESS
	clrf	DSKIO_SEC_3,ACCESS		; sector 0
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS	; one sector
	movwf	HOSTCMD,ACCESS			; read operation
	call	DSKIO_HDD
	sublw	0xcc
	bnz		DisplayReadErrAndRet

	lfsr	FSR0,STORM_HDDMBR-STORM_Pointers
	call	DisplaySector
	bc		DisplayHDD0Ret
	; move MBR to address 0x300
	lfsr	FSR0,0x100
	lfsr	FSR1,0x300
MBRMove0x300:
	movff	POSTINC0,POSTINC1
	movf	FSR0H,W,ACCESS
	sublw	0x03
	bnz		MBRMove0x300
	; initialize MBR pointer pointing to LVOL's OS type
	lfsr	FSR2,0x4be+4
	; clear MBR entry number
	clrf	ST_CHARMIN,BANKED

DisplayNextHDDEntry:
	; test if entry is empty
	movf	INDF2,W,ACCESS
	bnz		DisplayHDDEntry			; empty MBR entry

	movlw	0x10
	addwf	FSR2L,1,ACCESS			; shift MBR pointer to next record's OS type
	bra		DisplayNextHDDRecord

DisplayHDDEntry:
	; shift MBR pointer to the start sector of volume
	movlw	0x04
	addwf	FSR2L,1,ACCESS
	; read (boot) sector of volume
	movff	HDDSEL,DRVPRM			; restore hard drive ID
	movff	POSTINC2,DSKIO_SEC_0
	movff	POSTINC2,DSKIO_SEC_1
	movff	POSTINC2,DSKIO_SEC_2
	movff	POSTINC2,DSKIO_SEC_3	; set sector number of boot sector
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS	; one sector (HOSTCMD is already set)
	call	DSKIO_HDD
	sublw	0xcc
	bnz		DisplayReadErrAndRet
	; display this boot sector
	movff	HDDSEL,DRVPRM			; restore hard drive ID (modified during DSKIO_HDD)
	lfsr	FSR0,STORM_HDDBootSector-STORM_Pointers
	call	DisplaySector
	bc		DisplayHDD0Ret
	; shift MBR pointer to next record's OS type
	movlw	0x08
	addwf	FSR2L,1,ACCESS

DisplayNextHDDRecord:
	incf	ST_CHARMIN,1,BANKED		; increase MBR record count
	btfss	ST_CHARMIN,2,BANKED		; bit 2 of counter = 1 (counter=4)?
	bra		DisplayNextHDDEntry
	bra		DisplayHDD0Ret

; *** L3 displays contents of the boot sector of floppy disk in drive 0 (if present)
DisplayFDD0Boot:
	movlb	0x0f
	movf	NUMHFLP,1,BANKED
	bnz		DisplayFDD0DrivePresent

	; display that there's no drive 0 present
	lfsr	FSR0,STORM_NoFDD0Present-STORM_Pointers
DisplayFDD0NoC:						; (with C reset)
	call	DisplayFSR0
DisplayFDD0Ret:
	bcf		STATUS,C,ACCESS
	return

DisplayFDD0DrivePresent:
	; read boot sector into the buffer
	clrf	DRVPRM,ACCESS			; drive 0
	clrf	CURDRV,ACCESS			; reset current floppy drive number to 0
	clrf	DSKIO_SEC_0,ACCESS
	clrf	DSKIO_SEC_1,ACCESS		; sector 0
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS	; one sector
	movwf	HOSTCMD,ACCESS			; read operation
	call	DSKIO_FLOPPY
	call	WaitFDDMotorStop		; wait for FDD motor to stop
	sublw	0xcc
	bz		DisplayFDD0NoError

DisplayReadErrAndRet:
	; display "Read error..." and exti with C=0
	lfsr	FSR0,STORM_SectorReadError-STORM_Pointers
	bra		DisplayFDD0NoC

DisplayFDD0NoError:
	lfsr	FSR0,STORM_FDD0BootStr-STORM_Pointers
	; display sector with header in FSR0
	call	DisplaySector
	bra		DisplayFDD0Ret			; go clear C flag - do not exit higher level menu


; ***********************************************************************************************************
; *				Displays sector (512 bytes) with "press any key" between outputs and with header			*
; ***********************************************************************************************************
; in: FSR0 = header
DisplaySector:
	; set buffer decrement
	movlw	0x01
	movwf	SPDCOU,ACCESS
	; print header of the output
	call	DisplayFSR0
	; dump memory from 0x100 to 0x2ff
	lfsr	FSR1,0x100
	clrf	RECLENL,ACCESS			; 256 bytes, 16 lines by 16 bytes
	call	MCRAMOutput
	call	PressAnyKeyToContinue
	bc		DisplaySectorRet		; CTRL-BREAK breaks sequence
	clrf	RECLENL,ACCESS			; 256 bytes, 16 lines by 16 bytes
	call	MCRAMOutput
	call	PressAnyKeyToContinue
	bc		DisplaySectorRet		; CTRL-BREAK breaks sequence

	bcf		STATUS,C,ACCESS
DisplaySectorRet:
	return


; ***********************************************************************************************************
; *									Waits until FDDs motors stop											*
; ***********************************************************************************************************
; *** It is required as interrupts are disabled throughout STORM, and otherwise FDD motor will be steady
; rotating while STORM is running
WaitFDDMotorStop:
	; enable interrupts
	bsf		INTCON,GIE,ACCESS
	; preserve WREG
	movwf	TIMOUT2,ACCESS
	; wait for MOTAT0 to become 0
	movlw	0x10
	movwf	TIMOUT1,ACCESS			; set timeout, 16 times by 200ms (3.2 sec)
WaitFDD0MotorToStop:
	; delay for 200 ms
	movlw	0xc8
	call	Wait_ms
	; display "." to reset host dialogue timeout and show the progress
	movlw	"."
	call	SendCharToHost
	; poll status of the motor (0=stopped)
	movf	MOTATO,W,ACCESS
	iorwf	MOTBTO,W,ACCESS
	bz		FloppyMotorStopped		; if both motors' counters are 0s (both stopped)
	decfsz	TIMOUT1,1,ACCESS
	bra		WaitFDD0MotorToStop

	; display error message about motor not stopped
	lfsr	FSR0,STORM_FDD0MotorDidNotStop-STORM_Pointers
	call	DisplayFSR0

FloppyMotorStopped:
	; restore WREG
	movf	TIMOUT2,W,ACCESS
	; disable interrupts
	bcf		INTCON,GIE,ACCESS
	return


; ***********************************************************************************************************
; *								L3 END: displays MC buffer from 0x100 till 0xf00							*
; ***********************************************************************************************************
DisplayMCBuffer:
	lfsr	FSR0,STORM_BufferDump-STORM_Pointers
	call	DisplayFSR0
	; set buffer decrement
	clrf	SPDCOU,ACCESS

	lfsr	FSR1,0x100				; set start to the 0x100 (beginning of work buffer)
DisplayMCBufferLoop:
	; display 256 bytes
	clrf	RECLENL,ACCESS			; 256 bytes, 16 lines by 16 bytes
	call	MCRAMOutput
	call	PressAnyKeyToContinue
	bc		EndOfDisplayMCBuffer

	movf	FSR1H,W,ACCESS
	sublw	0x0f
	bnz		DisplayMCBufferLoop

EndOfDisplayMCBuffer:
	bcf		STATUS,C,ACCESS			; reset C flag - do not exit to parent menu
	return


; ***********************************************************************************************************
; *								Displays "Press any key..." and wait for keypress							*
; ***********************************************************************************************************
PressAnyKeyToContinue:
	; display "press any key to continue..."
	lfsr	FSR0,STORM_PressContinue-STORM_Pointers
	call	DisplayFSR0
	; wait for keypress
	movlw	0x01
	call	SendCharToHost				; request host to read character
	call	WaitSTORMInput
	return								; preserves C flag (CTRL-BREAK identifier)


; ***********************************************************************************************************
; *									L3 END: Displays MC workset (registers)									*
; ***********************************************************************************************************
DisplayMCWorkset:
	lfsr	FSR0,STORM_MemoryDump-STORM_Pointers
	call	DisplayFSR0
	; set buffer decrement
	clrf	SPDCOU,ACCESS

	; dump memory from 0x020 to 0x05f
	movlw	0x40
	movwf	RECLENL,ACCESS				; number of bytes (64)
	lfsr	FSR1,0x020					; FSR1 start at 0x020
	call	MCRAMOutput
	; dump memory from 0xe00 to 0xe20
	movlw	0x20
	movwf	RECLENL,ACCESS				; number of bytes (32)
	lfsr	FSR1,0xe00					; FSR1 start at 0xe00
	call	MCRAMOutput
	; dump memory from 0xf00 to 0xf3f
	movlw	0x40
	movwf	RECLENL,ACCESS				; number of bytes (64)
	lfsr	FSR1,0xf00					; FSR1 start at 0xf00
	call	MCRAMOutput
	bcf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *									Performs HEX/Character RAM output in table								*
; ***********************************************************************************************************
; in: FSR0 = buffer address to display
; in: RECLENL = number of bytes to display (divisible by 16), 0x00 means 256 bytes
; in: SPDCOU = FSR high address decrement (e.g. reads from address 0x100 but displays 0x000 if SPDCOU=0x01)
MCRAMOutput:
	movf	FSR1L,W,ACCESS
	andlw	0x0f						; each 16th byte?
	bnz		DoNotOutputStringNum

	lfsr	FSR0,STORM_CR-STORM_Pointers
	call	DisplayFSR0					; display CR
	swapf	FSR1L,W,ACCESS
	andlw	0xf
	movwf	FTMP,BANKED
	movf	SPDCOU,W,ACCESS
	subwf	FSR1H,W,ACCESS
	swapf	WREG,1,ACCESS
	andlw	0xf0
	iorwf	FTMP,W,BANKED				; now byte contains bits 23..8 of the FSR1
	call	SendHEXToHost
	lfsr	FSR0,STORM_Zero-STORM_Pointers
	call	DisplayFSR0

DoNotOutputStringNum:
	movf	POSTINC1,W,ACCESS
	call	SendHEXToHost
	movlw	0x20
	call	SendCharToHost				; space after each hex

	movf	FSR1L,W,ACCESS
	andlw	0x03						; each 4th byte?
	bnz		NoAddSpace
	movlw	0x20
	call	SendCharToHost				; space after each 4th byte
NoAddSpace:
	movf	FSR1L,W,ACCESS
	andlw	0x0f						; each 16th byte?
	bnz		NoSymbolicalOutput

	; display 16 characters, replacing chars < 0x20 and >= 0xfe with '.'
	movlw	0x10
	movwf	RECLENH,ACCESS				; counter
	subwf	FSR1L,1,ACCESS
	movlw	0x00
	subwfb	FSR1H,1,ACCESS				; move FSR1 pointer 16 bytes back

CharDisplayLoop:
	movf	INDF1,W,ACCESS
	sublw	0x1f
	bc		ReplaceWithDot				; replace with "." if < 0x20
	addlw	0xde
	bnc		ReplaceWithDot				; and if >=0xfe
	sublw	0x7e
	bnz		DoNotReplaceWithDot			; and if equal to 0x7f (DEL)
ReplaceWithDot:
	movf	POSTINC1,W,ACCESS			; shift FSR1 pointer
	movlw	"."
	bra		SendSymbolToHost
DoNotReplaceWithDot:
	movf	POSTINC1,W,ACCESS
SendSymbolToHost:
	call	SendCharToHost
	decfsz	RECLENH,1,ACCESS
	bra		CharDisplayLoop

NoSymbolicalOutput:
	decfsz	RECLENL,1,ACCESS
	bra		MCRAMOutput
	return


; ***********************************************************************************************************
; *						Prints string offsetted by FSR0 with control chars and variables					*
; ***********************************************************************************************************
DisplayFSR0:
	; set ROM pointer to the respective data in ROM
	; adding offset represented by FSR0 to the table base
	movlw	LOW STORM_Pointers
	addwf	FSR0L,W,ACCESS
	movwf	TBLPTRL,ACCESS
	movlw	HIGH STORM_Pointers
	addwfc	FSR0H,W,ACCESS
	movwf	TBLPTRH,ACCESS

TableFSRNextRead:
	; read string char and checking if we are at the end or it is variable
	tblrd*+
	tstfsz	TABLAT,ACCESS
	bra		NextFSRLocation
	return

NextFSRLocation:
	; return string char to the host
	movlw	0x7f
	cpfsgt	TABLAT,ACCESS
	bra		NotControlChar

	push
	movf	TBLPTRL,W,ACCESS
	movwf	TOSL,ACCESS
	movf	TBLPTRH,W,ACCESS
	movwf	TOSH,ACCESS					; preserve table pointer H:L, pointer to the string
	push
	movlw	LOW RetAddr
	movwf	TOSL,ACCESS
	movlw	HIGH RetAddr
	movwf	TOSH,ACCESS
	push								; save return address to the stack
	; point flash table pointer to the calling table
	movlw	LOW CallTable
	movwf	TBLPTRL,ACCESS
	movlw	HIGH CallTable
	movwf	TBLPTRH,ACCESS
	; adding offset
	movlw	0x80
	subwf	TABLAT,W,ACCESS				; number of sub to call
	addwf	WREG,W,ACCESS				; *2
	addwf	TBLPTRL,1,ACCESS
	movlw	0x00
	addwfc	TBLPTRH,1,ACCESS
	tblrd*+
	movf	TABLAT,W,ACCESS
	movwf	TOSL,ACCESS
	tblrd*
	movf	TABLAT,W,ACCESS
	movwf	TOSH,ACCESS
	clrf	TOSU,ACCESS
	return								; call specific routine by address in TOS
RetAddr:
	movff	TOSL,TBLPTRL				; stack already popped to the saved table pointer level
	movff	TOSH,TBLPTRH				; restore flash table pointer H:L
	pop
	bra		TableFSRNextRead

NotControlChar:
	; send sync char to the host
	movlw	0xAE
	call	WaitForHostRead				; return 0xAB to the host - command has started
	btfss	STATUS,C,ACCESS				; A0 address line state during host request
	goto	NoCommandToRun				; access to port base+1 is not assumed this time, aborting MC command

	movf	TABLAT,W,ACCESS
	call	TestForEndStrChar
	call	WaitForHostRead
	btfss	STATUS,C,ACCESS				; A0 address line state during host request
	goto	HostWaitMainLoop			; access to port base+1 is not assumed this time, aborting MC command

	bra		TableFSRNextRead


; ***********************************************************************************************************
; *					Subroutined which are called during Operation mode settings display						*
; ***********************************************************************************************************
FloppyDisableEnable:
	lfsr	FSR0,STORM_Disabled-STORM_Pointers
	btfsc	OPMODE,6,BANKED
	lfsr	FSR0,STORM_Enabled-STORM_Pointers
	call	DisplayFSR0
	return
HDDDisableEnable:
	lfsr	FSR0,STORM_Disabled-STORM_Pointers
	btfsc	OPMODE,7,BANKED
	lfsr	FSR0,STORM_Enabled-STORM_Pointers
	call	DisplayFSR0
	return
DeviceOrderStr:
	lfsr	FSR0,STORM_HDDFirst-STORM_Pointers
	btfsc	OPMODE,2,BANKED
	lfsr	FSR0,STORM_FloppyFirst-STORM_Pointers
	call	DisplayFSR0
	return
FDDModeDisplay:
	lfsr	FSR0,STORM_Standard-STORM_Pointers
	btfsc	OPMODE,0,BANKED
	lfsr	FSR0,STORM_Compatible-STORM_Pointers
	call	DisplayFSR0
	return
PhantomModeDisplay:
	lfsr	FSR0,STORM_Disabled-STORM_Pointers
	btfss	OPMODE,1,BANKED
	lfsr	FSR0,STORM_Enabled-STORM_Pointers
	call	DisplayFSR0
	return
DisplaySpT:
	movf	DSKIO_SpT,W,ACCESS
	call	SendHEXToHost
	return
DisplayHeads:
	movf	DSKIO_Heads,W,ACCESS
	call	SendHEXToHost
	return
DisplaySecFat:
	movf	DIO_NumSecProcessed,W,ACCESS	; DIO_NumSecProcessed is temp used as sectors-per-FAT value
	bnz		SecFatNot100h
	movlw	0x01
	call	SendHEXToHost					; display 01 at the beginning of SpF in case it is 0
	movf	DIO_NumSecProcessed,W,ACCESS	; so that it look "0100h"
SecFatNot100h:
	call	SendHEXToHost
	return


; ***********************************************************************************************************
; *									Display maximal cluster number											*
; ***********************************************************************************************************
DisplayMaxCluster:
	call	CalcMaxCluster
	; check for max number of clusters = 0 and add "1" at the front to the output
	movf	DVDT_0,W,BANKED
	iorwf	DVDT_1,W,BANKED					; both bytes of the word are 0s? (10000h?)
	movlw	0x01
	btfsc	STATUS,Z,ACCESS
	call	SendHEXToHost					; yes, display "01" before zeros to indicate 65536 clusters

	; display max number of clusters
	movf	DVDT_1,W,BANKED
	call	SendHEXToHost
	movf	DVDT_0,W,BANKED
	call	SendHEXToHost					; print number of clusters
	return


; ***********************************************************************************************************
; *					Calculates max cluster number basing on the partition type and SpF						*
; ***********************************************************************************************************
; in: ST_PRTTYP, DIO_NumSecProcessed
; out: ST_MAXCLUx
CalcMaxCluster:
	; calculate maximal cluster count with specific number of FAT
	; sectors for layout indicated in ST_PRTTYP
	movff	ST_PRTTYP,WREG
	movf	WREG,W,ACCESS
	bnz		MaxClusPartitionFAT16			; ST_PRTTYP!=0 then FAT16

	; we are here for calculation of FAT12 partition type
	; Number of clusters = FAT sectors*512*2/3
	rlcf	DIO_NumSecProcessed,W,ACCESS	; sectors per FAT*2
	rlcf	WREG,W,ACCESS					; sectors per FAT*4
	movwf	DVDT_1,BANKED
	clrf	DVDT_0,BANKED					; *256
	movlw	0x03
	movwf	DVSR_0,BANKED
	clrf	DVSR_1,BANKED
	call	DIV16							; /3
	bra		SetMaxCluster

MaxClusPartitionFAT16:
	movff	DIO_NumSecProcessed,DVDT_1
	clrf	DVDT_0,BANKED					; sectors per FAT*512/2 (=0000h for 65536 clusters)

SetMaxCluster:
	; preserve number of clusters available in FAT into ST_MAXCLUx
	movff	DVDT_0,ST_MAXCLU0
	movff	DVDT_1,ST_MAXCLU1
	return

; ***********************************************************************************************************
; *									Display sectors per cluster variable									*
; ***********************************************************************************************************
DisplaySecClus:
	movf	DSKIO_NumOfSec,W,ACCESS			; DSKIO_NumOfSec is temp used as sectors-per-cluster value
	call	SendHEXToHost
	return


; ***********************************************************************************************************
; *										Display estimated volume size										*
; ***********************************************************************************************************
DisplaySize:
	movff	ST_MAXCLU0,DVDT_0
	movff	ST_MAXCLU1,DVDT_1
	decf	DVDT_0,1,BANKED
	bc		NODVDTC
	decf	DVDT_1,1,BANKED					; decrement count of clusters
NODVDTC:
	movff	DSKIO_NumOfSec,DVSR_0
	clrf	DVSR_1,BANKED
	call	MUL16							; max number of clusters * sectors-per-cluster (32-bit value)

	; convert from sectors to KBytes and display size
	call	ConvertSectorsToKBandDisplay
	return


; ***********************************************************************************************************
; *						Convert size in sectors in RMDR to KBytes, copy to DVDT and display					*
; ***********************************************************************************************************
ConvertSectorsToKBandDisplay:
	rrcf	RMDR_3,W,BANKED
	andlw	0x7f					; does not affect C flag
	movwf	DVDT_3,BANKED
	rrcf	RMDR_2,W,BANKED
	movwf	DVDT_2,BANKED
	rrcf	RMDR_1,W,BANKED
	movwf	DVDT_1,BANKED
	rrcf	RMDR_0,W,BANKED
	movwf	DVDT_0,BANKED
	call	CorrectDVDTForSize
	call	SendCharToHost
	movlw	"B"
	call	SendCharToHost
	return


; ***********************************************************************************************************
; *									Displays partition number to configure									*
; ***********************************************************************************************************
DisplayPartitionNum:
	movff	ST_PRTCNF,WREG
	call	SendHEXToHost
	return


; ***********************************************************************************************************
; *					Returns C flag set as a signal to the menu to exit to the higher level					*
; ***********************************************************************************************************
ReturnCarryFlag:
	bsf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *									Prints string in FSR0 and gets "Y" prompt								*
; ***********************************************************************************************************
; in: FSR0 = relative pointer to the string to print (should end with 0xc0)
; out: Z=1 if upper case ("Y") was pressed, C=1 if CTRL-BREAK
PromptYN:
	call	DisplayFSR0
	call	WaitSTORMInput
	bc		ReturnCarryFlag				; finish output if host signalled CTRL-BREAK
	call	SendCharToHost				; display typed character
	sublw	"Y"
	bcf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *								Displays HDD device number selected in DRVPRM								*
; ***********************************************************************************************************
DisplayHDDDRVPRMNum:
	movf	DRVPRM,W,ACCESS
DisplayCharPlus30:
	addlw	0x30
	call	SendCharToHost
	return


; ***********************************************************************************************************
; *								Displays number of MBR entry from ST_CHARMIN								*
; ***********************************************************************************************************
DisplayMBRVolumeEntry:
	movf	ST_CHARMIN,W,BANKED
	bra		DisplayCharPlus30


; ***********************************************************************************************************
; *					Displays file system type trailing character: 2 for FAT12 and 6 for FAT16				*
; ***********************************************************************************************************
DisplayFSType:
	movlw	0x02					; "2"
	movlb	0xe
	tstfsz	ST_PRTTYP,BANKED
	movlw	0x06					; "6"
	movlb	0xf
	bra		DisplayCharPlus30

; ***********************************************************************************************************
; *									Displays number of directory entries									*
; ***********************************************************************************************************
DisplayDirEnt:
	movff	ST_DIRENT1,WREG
	call	SendHEXToHost
	movff	ST_DIRENT0,WREG
	call	SendHEXToHost
	return


; ***********************************************************************************************************
; *					Prompt for hard disk number to display and display partition table						*
; ***********************************************************************************************************
; out: HDDSEL, DSKIO_SpT, DSKIO_Heads
; out: C=1 if aborted/no hard disks/error reading MBR
SelectHDDandListPartitions:
	movlb	0x0f
	movf	NUMHDD,W,BANKED
	bnz		SelectHDD_HDDInstalled		; number of HDDs enumerated != 0?

	; display "No HHDs installed" and return
	lfsr	FSR0,STORM_NoHDDInstalled-STORM_Pointers

SelectHDDRetFSR:
	call	DisplayFSR0
SelectHDDReturn:
	bsf		STATUS,C,ACCESS				; set C flag
	return

SelectHDD_HDDInstalled:
	; print HDDs in loop
	clrf	DRVPRM,ACCESS				; clear HDD counter

SelectHDD_PrintNextHDD:
	lfsr	FSR0,STORM_HDDList-STORM_Pointers
	call	DisplayFSR0

	incf	DRVPRM,1,ACCESS
	movf	DRVPRM,W,ACCESS
	subwf	NUMHDD,W,BANKED
	bnz		SelectHDD_PrintNextHDD

	; display prompt
	lfsr	FSR0,STORM_HDDPrompt-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movf	NUMHDD,W,BANKED
	addlw	0x2f
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		SelectHDDReturn				; return with C=1

	movwf	DRVPRM,ACCESS				; HDD target device is identified
	movwf	HDDSEL,BANKED				; preserve currently selected hard drive

	clrf	DSKIO_SEC_0,ACCESS
	clrf	DSKIO_SEC_1,ACCESS
	clrf	DSKIO_SEC_2,ACCESS
	clrf	DSKIO_SEC_3,ACCESS			; sector 0 - boot sector of LVOL (logical sector starts with 0)
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 1
	movwf	HOSTCMD,ACCESS				; read operation
	call	DSKIO_HDD					; read physical harddrive
	movlb	0x0f
	movwf	FTMP,BANKED					; preserve return code
	sublw	0xCC
	bz		SelectHDD_ReadMBROk

	; display "Error reading sector"
	lfsr	FSR0,STORM_SectorReadError-STORM_Pointers
	bra		SelectHDDRetFSR

SelectHDD_ReadMBROk:
	; display header of the partition list
	lfsr	FSR0,STORM_PartitionList-STORM_Pointers
	call	DisplayFSR0

	lfsr	FSR1,0x2be					; (FSR1) start of partition table loaded into MC buffer
	movlb	0x0f
	movlw	0x04
	movwf	FTMP,BANKED

	; now print all entries of the partition table (including empty)
PartitionTablePrintLoop:
	movf	FTMP,W,BANKED
	sublw	0x04
	addlw	0x30
	call	SendCharToHost
	movlw	0x20
	call	SendCharToHost				; partition number and space character

	; print partition activity status and OS type with following space char
	call	InitHDD_DisplayPartitionAOS

	; now print partition start LBA and sector count (though space char)
	movlw	0x07
	addwf	FSR1L,1,ACCESS				; shift to start LBA: last memory cell for POSTDEC1
	movlw	0x08						; output count
	movwf	SPDCOU,ACCESS				; using SPDCOU as a counter

PartEntryPrintSizesLoop:
	movlw	0x04
	subwf	SPDCOU,W,ACCESS				; between portions of output?
	bnz		SkipSpaceChar

	movlw	0x08
	addwf	FSR1L,1,ACCESS				; shift to count LBA: last memory cell+1 for POSTDEC1
	movlw	0x20
	call	SendCharToHost				; yes, between, print space char

SkipSpaceChar:
	movf	POSTDEC1,W,ACCESS
	call	SendHEXToHost

	decfsz	SPDCOU,1,ACCESS
	bra		PartEntryPrintSizesLoop

	movlw	0x05
	addwf	FSR1L,1,ACCESS				; shift to next record after serie of POSTDEC1

	; display CR/LF
	lfsr	FSR0,STORM_CR-STORM_Pointers
	call	DisplayFSR0

	decfsz	FTMP,1,BANKED
	bra		PartitionTablePrintLoop

	bcf		STATUS,C,ACCESS				; clear C flag, no error, proceed further
	return


; ***********************************************************************************************************
; *							Reinitialize LVOL's control structures (quick format)							*
; ***********************************************************************************************************
ReInitializeLVOL:
	; set starting sector for DSKIO_HDD and configure partition/boot sector information
	call	SetFisrtSectorAndConfigPartition

	; setting the flag that vital changes to storage subsystem were performed
	setf	CFGCHGFLG,BANKED			; no vital config changes were made so far

	; calculate FAT sector address in FSR2
	lfsr	FSR2,0x300
	movff	ST_PRTCNF,WREG
	addwf	WREG,W,ACCESS
	addwf	FSR2H,1,ACCESS				; now points to respective boot sector location
	; write LVOL structures
	call	WriteLVOLStructures
	btfsc	STATUS,C,ACCESS
	goto	PartMgr_WriteSectorError	; sector write error message - and exit to the menu
	; return no error
	bcf		STATUS,C,ACCESS
	return


; ***********************************************************************************************************
; *			Set starting sector for DSKIO_HDD and configure partition/boot sector information				*
; ***********************************************************************************************************
; in: MBR is located at address 0x100
; in: pointer to the LVOL's MBR structure (to OS type) is in FSR1
SetFisrtSectorAndConfigPartition:
	movlw	0x04
	addwf	FSR1L,1,ACCESS				; now MBR points to the LVOL start LBA
	movff	POSTINC1,DSKIO_SEC_0
	movff	POSTINC1,DSKIO_SEC_1
	movff	POSTINC1,DSKIO_SEC_2
	movff	POSTINC1,DSKIO_SEC_3		; set boot sector number to write to

	; all required values are set previously
	call	ConstructBootSector			; fill boot sector in
	return


; ***********************************************************************************************************
; *								Recover the boot sector and first FAT's signature							*
; ***********************************************************************************************************
; in: MBR is located at address 0x100
; in: FSR1=pointer to the LVOL's MBR structure (to OS type)
RecoverLVOLBootAndFAT:
	; set starting sector for DSKIO_HDD and configure partition/boot sector information
	call	SetFisrtSectorAndConfigPartition

	; setting the flag that vital changes to storage subsystem were performed
	setf	CFGCHGFLG,BANKED			; no vital config changes were made so far

	; calculate FAT sector address and move it to 0x100
	lfsr	FSR0,0x300
	movff	ST_PRTCNF,WREG
	addwf	WREG,W,ACCESS
	addwf	FSR0H,1,ACCESS				; now points to respective boot sector location
	lfsr	FSR1,0x100
RecoverLVOLMoveBoot:
	movff	POSTINC0,POSTINC1
	movf	FSR1H,W,ACCESS
	sublw	0x03
	bnz		RecoverLVOLMoveBoot

	; configure DSKIO_HDD parameters, and write boot sector
	movff	HDDSEL,DRVPRM
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 1 (boot only)
	movlw	0x02
	movwf	HOSTCMD,ACCESS				; write operation = 2
	call	DSKIO_HDD
	sublw	0xcc
	btfss	STATUS,Z,ACCESS
	goto	PartMgr_WriteSectorError	; sector write error message - and exit to the menu

	; now read first FAT sector (DSKIO_SEC_x points to next sector)
	movff	HDDSEL,DRVPRM
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 1
	movwf	HOSTCMD,ACCESS				; read operation = 1
	call	DSKIO_HDD
	sublw	0xcc
	btfss	STATUS,Z,ACCESS
	goto	SectorReadErrorDisplay		; sector read error message - and exit to the menu

	; decrease sector number to point back to previous sector
	decf	DSKIO_SEC_0,1,ACCESS
	movlw	0x00
	subwfb	DSKIO_SEC_1,1,ACCESS
	subwfb	DSKIO_SEC_2,1,ACCESS
	subwfb	DSKIO_SEC_3,1,ACCESS

	; recoving FAT signature
	lfsr	FSR0,0x100
	movlw	0xf0
	movwf	POSTINC0,ACCESS				; media descriptor
	setf	POSTINC0,ACCESS
	setf	POSTINC0,ACCESS				; cluster 0/1 (for FAT12)
	movff	ST_PRTTYP,WREG				; partition type
	tstfsz	WREG,ACCESS					; partition type = 0? (FAT12)
	setf	POSTINC0,ACCESS				; no, it is FAT16, put another 0xFF for second half of cluster 2

	; writing FAT back
	movff	HDDSEL,DRVPRM
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 1
	movlw	0x02
	movwf	HOSTCMD,ACCESS				; write operation = 2
	call	DSKIO_HDD
	sublw	0xcc
	btfss	STATUS,Z,ACCESS
	goto	PartMgr_WriteSectorError	; sector write error message - and exit to the menu

	; done
	bra		ActDeactReturn


; ***********************************************************************************************************
; *											Initialize LVOL menu											*
; ***********************************************************************************************************
InitLVOL:
	; select HDD
	call	SelectHDDandListPartitions
	bc		ActDeactReturn				; selection calcelled

	; display "Enter volume number to initialize"
	lfsr	FSR0,STORM_LVOL_Initialize-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x33
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		ActDeactReturn				; return with C=1

	movff	WREG,ST_PRTCNF				; save selected LVOL number (defines boot sector location in memory)
	lfsr	FSR1,0x2be+4				; address +4 = volume OS type
	swapf	WREG,W,ACCESS				; volume # parameter * 16
	addwf	FSR1L,1,ACCESS				; move pointer to the start of respective
										; LVOL record in MBR (FSR1H is not affected)
	movf	INDF1,W,ACCESS
	bnz		InitLVOL_not_empty

	; display "LVOL is empty"
	lfsr	FSR0,STORM_LVOLEmpty-STORM_Pointers
	call	DisplayFSR0
	bra		ActDeactReturn

InitLVOL_not_empty:
	; prepare boot sectors
	call	PrepareBootSectors			; four sectors, at 0x300, 0x500, 0x700 and 0x900
	; set default volume information
	call	SetParametersFAT12
	decfsz	INDF1,W,ACCESS				; OS type-1 (=0 if FAT12)?
	call	SetParametersFAT16			; OS type not 0x01 = FAT16

	lfsr	FSR2,0x180					; offset to LVOL parameter recovery table in MBR
	movff	ST_PRTCNF,WREG				; LVOL selection
	addwf	WREG,W,ACCESS
	addwf	WREG,W,ACCESS
	addwf	WREG,W,ACCESS				; *8
	addwf	FSR2L,1,ACCESS				; now points to respective recovery table
	; restore vital LVOL configuration parameters
	movff	POSTINC2,DSKIO_NumOfSec			; sectors per cluster
	movff	POSTINC2,DIO_NumSecProcessed	; sectors in FAT copy

	; calculate maximal cluster number (can only happen after initialization
	; of DSKIO_NumOfSec (SpC) and DIO_NumSecProcessed (SpF)
	call	CalcMaxCluster				; ST_MAXCLU1:ST_MAXCLU0

	; print string of configurtion info "Fixed parameters... Configurable parameters"
	; and list menu (recover/re-init/exit)
	; during print respective subs are called and values calculated
	lfsr	FSR0,STORM_RecoverLVOL-STORM_Pointers
;	; display "LVOL Initialization/recovery information:" and get option
;	lfsr	FSR0,STORM_LVOL_Initialize-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x32
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		ActDeactReturn				; return with C=1

	lfsr	FSR2,CallTableLVOLInit-STORM_Pointers
	call	CallSubInW
	bra		ActDeactReturn				; any option - return back to parent menu


; ***********************************************************************************************************
; *										Change partition activation status									*
; ***********************************************************************************************************
ChangePartitionActivation:
	; select HDD
	call	SelectHDDandListPartitions
	bnc		ActDeactHDDWasSelected

ActDeactReturn:
	bcf		STATUS,C,ACCESS				; clear C flag = do not get to higher level menu
	return

ActDeactHDDWasSelected:
	; now we have hard disk drive selected in HDDSEL, and
	; MBR loaded at 0x100

	; display "Enter volume number to toggle"
	lfsr	FSR0,STORM_LVOL_Act_Deact-STORM_Pointers
	movlw	0x30
	movwf	ST_CHARMIN,BANKED
	movlw	0x33
	movwf	ST_CHARMAX,BANKED			; Set MIN and MAX
	call	GetCharAndValidate			; get char
	bc		ActDeactReturn				; return with C=1

	lfsr	FSR0,0x2be
	swapf	WREG,W,ACCESS				; volume # parameter * 16
	addwf	FSR0L,1,ACCESS				; move pointer to the start of respective
										; LVOL record in MBR (FSR1H is not affected)
	btg		INDF0,7,ACCESS				; toggle activation bit (bit 7 at offest +0)
	; request confirmation
	lfsr	FSR0,STORM_LVOL_AD_Confirm-STORM_Pointers
	call	PromptYN
	bz		ActDeactMBRWriteConfirmed
	; not Y - display "Aborted" and exit
	goto	PartMgrAbortedExit

ActDeactMBRWriteConfirmed:
	; setting the flag that vital changes to storage subsystem were performed
	setf	CFGCHGFLG,BANKED			; no vital config changes were made so far
	; writing MBR
	movff	HDDSEL,DRVPRM				; restore drive ID
	clrf	DSKIO_SEC_0,ACCESS
	clrf	DSKIO_SEC_1,ACCESS
	clrf	DSKIO_SEC_2,ACCESS
	clrf	DSKIO_SEC_3,ACCESS			; sector 0 - boot sector of LVOL (logical sector starts with 0)
	movlw	0x01
	movwf	DSKIO_NumOfSec,ACCESS		; number of sectors = 1
	movlw	0x02
	movwf	HOSTCMD,ACCESS				; write operation
	call	DSKIO_HDD					; read physical harddrive
	sublw	0xCC
	bz		ActDeactReturn				; successful write
	goto	PartMgr_WriteSectorError	; unsuccessful write: display "write error"

;********************************************************************************
;*			END OF: GR8STORM_MANAGER.INC										*
;********************************************************************************