;********************************************************************************
;*																				*
;*								License and T&C									*
;*																				*
;*			GR8BIT storage subsystem microcontroller microcode version 3.0		*
;*			     This software is copyleft 2011-12 by Eugeny Brychkov			*
;*																				*
;* You can use and modify its source and executable, given two conditions are	*
;* met: (a) you keep this Copyleft statement in the source code, and you do		*
;* not distribute software and resulting executables on the commercial basis	*
;* and for the purposes, not related to GR8BIT platform, without written		*
;* permission of Eugeny Brychkov (eugeny_brychkov@hotmail.com)					*
;*																				*
;********************************************************************************
;*			File name: GR8STORM_MATH.INC										*
;********************************************************************************

; This file represents 16- and 32-bit math routines
GR8STORM_MATH	CODE


; ***********************************************************************************************************
; *						Multiply two 16 bit variables (words), result is 32-bit								*
; ***********************************************************************************************************
; in: DVDT=argument1, DVSR=argument2, both 16-bit
; out: RMDR=result, 32-bit
; arguments are located in the page 15
MUL16:
	movlb	0x0f
	clrf	DVDT_3,BANKED
	clrf	DVDT_2,BANKED
	clrf	DVSR_3,BANKED
	clrf	DVSR_2,BANKED
	movlw	0x10				; 16 bits
	movwf	DIVCTR,BANKED
	call	MULX
	return


; ***********************************************************************************************************
; *						Multiply two 32 bit variables (dwords), result is 32-bit							*
; ***********************************************************************************************************
; in: DVDT=argument1, DVSR=argument2, both 32-bit
; out: RMDR=result, 32-bit
; arguments are located in the page 15
MUL32:
	movlb	0x0f
	movlw	0x20				; 32 bits
	movwf	DIVCTR,BANKED
	call	MULX
	return

; Runs multiplication according to parameters previously set
MULX:
	clrf	RMDR_0,BANKED
	clrf	RMDR_1,BANKED
	clrf	RMDR_2,BANKED
	clrf	RMDR_3,BANKED		; clear result

MULX_Cycle:
	rrcf	DVDT_3,1,BANKED
	rrcf	DVDT_2,1,BANKED
	rrcf	DVDT_1,1,BANKED
	rrcf	DVDT_0,1,BANKED		; rotate divident right to C
	bnc		MULX_NoAdd

	; adding DVSR to RMDR
	call	Add32_RMDR_eq_DVSR_plus_RMDR
MULX_NoAdd:
	bcf		STATUS,C,ACCESS
	rlcf	DVSR_0,1,BANKED
	rlcf	DVSR_1,1,BANKED
	rlcf	DVSR_2,1,BANKED
	rlcf	DVSR_3,1,BANKED		 ; rotate divisor/multiplier left
	decfsz	DIVCTR,1,BANKED
	bra		MULX_Cycle
	return


; ***********************************************************************************************************
; *						Divide two 32 bit variables (dwords), result is 32-bit								*
; ***********************************************************************************************************
; in: DVDT=dividend, DVSR=divisor
; out: DVDT=result, RMDR=remainder
; arguments are located in the page 15
DIV32:
	movlb	0x0f
	clrf	RMDR_0,BANKED
	clrf	RMDR_1,BANKED
	clrf	RMDR_2,BANKED
	clrf	RMDR_3,BANKED		; clear remainder

	movlw	0x20				; 32 bits
	movwf	DIVCTR,BANKED
	rlcf	DVDT_0,1,BANKED
	rlcf	DVDT_1,1,BANKED
	rlcf	DVDT_2,1,BANKED
	rlcf	DVDT_3,1,BANKED		; rotate divident left to C
DIV32_Loop:
	rlcf	RMDR_0,1,BANKED		; rotate C into LSB of remainder
	rlcf	RMDR_1,1,BANKED
	rlcf	RMDR_2,1,BANKED
	rlcf	RMDR_3,1,BANKED		; rotate remainder left to C
	bc		DIV32_substract

	call	DIV32_subs_rmdr_dvsr_nc
	bnc		DIV32_NoAdd

	; now add it back - substraction result was negative
	movf	DVSR_0,W,BANKED
	addwf	RMDR_0,1,BANKED
	movf	DVSR_1,W,BANKED
	addwfc	RMDR_1,1,BANKED	
	movf	DVSR_2,W,BANKED
	addwfc	RMDR_2,1,BANKED
	movf	DVSR_3,W,BANKED
	addwfc	RMDR_3,1,BANKED

DIV32_NoAdd:
	btg		STATUS,C,ACCESS		; invert C flag

DIV32_shift_divident:
	rlcf	DVDT_0,1,BANKED
	rlcf	DVDT_1,1,BANKED
	rlcf	DVDT_2,1,BANKED
	rlcf	DVDT_3,1,BANKED		; rotate divident left to C
	decfsz	DIVCTR,1,BANKED
	bra		DIV32_Loop
	return

DIV32_substract:
	call	DIV32_subs_rmdr_dvsr_nc
	bra		DIV32_shift_divident

DIV32_subs_rmdr_dvsr_nc:
	; substract divisor from remainder, returns C flag
	movf	DVSR_0,W,BANKED
	subwf	RMDR_0,1,BANKED
	movf	DVSR_1,W,BANKED
	subwfb	RMDR_1,1,BANKED
	movf	DVSR_2,W,BANKED
	subwfb	RMDR_2,1,BANKED
	movf	DVSR_3,W,BANKED
	subwfb	RMDR_3,1,BANKED
	btg		STATUS,C,ACCESS			; toggle C flag bit as it is inversed after substraction op
	return


; ***********************************************************************************************************
; *						Divide two 16 bit variables (words), result is 16-bit								*
; ***********************************************************************************************************
; in: DVDT=dividend, DVSR=divisor
; out: DVDT=result, RMDR=remainder
; arguments are located in the page 15
DIV16:
	movlb	0x0f				; page 15
	clrf	RMDR_0,BANKED
	clrf	RMDR_1,BANKED		; clear remainder

	movlw	0x10				; 16 bits
	movwf	DIVCTR,BANKED
	rlcf	DVDT_0,1,BANKED
	rlcf	DVDT_1,1,BANKED		; rotate divident left to C
DIV16_Loop:
	rlcf	RMDR_0,1,BANKED		; rotate C into LSB of remainder
	rlcf	RMDR_1,1,BANKED		; rotate remainder left to C
	bc		DIV16_substract
	call	DIV16_subs_rmdr_dvsr_nc
	bnc		DIV16_NoAdd

	; now add with no carry
	call	Add16_RMDR_eq_DVSR_plus_RMDR

DIV16_NoAdd:
	btg		STATUS,C,ACCESS		; invert C flag

DIV16_shift_divident:
	rlcf	DVDT_0,1,BANKED
	rlcf	DVDT_1,1,BANKED		; rotate divident left to C
	decfsz	DIVCTR,1,BANKED
	bra		DIV16_Loop
	return

DIV16_substract:
	call	DIV16_subs_rmdr_dvsr_nc
	bra		DIV16_shift_divident

DIV16_subs_rmdr_dvsr_nc:
	; substract divisor from remainder, returns C flag
	movf	DVSR_0,W,BANKED
	subwf	RMDR_0,1,BANKED
	movf	DVSR_1,W,BANKED
	subwfb	RMDR_1,1,BANKED	
	btg		STATUS,C,ACCESS			; toggle C flag bit as it is inversed after substraction op
	return

Add16_RMDR_eq_DVSR_plus_RMDR:
	movf	DVSR_0,W,BANKED
	addwf	RMDR_0,1,BANKED
	movf	DVSR_1,W,BANKED
	addwfc	RMDR_1,1,BANKED	
	return

Add32_RMDR_eq_DVSR_plus_RMDR:
	movf	DVSR_0,W,BANKED
	addwf	RMDR_0,1,BANKED
	movf	DVSR_1,W,BANKED
	addwfc	RMDR_1,1,BANKED	
	movf	DVSR_2,W,BANKED
	addwfc	RMDR_2,1,BANKED	
	movf	DVSR_3,W,BANKED
	addwfc	RMDR_3,1,BANKED	
	return

;********************************************************************************
;*			END OF: GR8STORM_MATH.INC											*
;********************************************************************************